###
## * << Haru Free PDF Library 2.0.0 >> -- encoding_list.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *
from haru.hpdf_errorcode import *


@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: %s, detail_no=%u\n", error_detail[error_no],
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)


PAGE_WIDTH = 420
PAGE_HEIGHT = 400
CELL_WIDTH = 20
CELL_HEIGHT = 20
CELL_HEADER = 10

def draw_graph (page):

    # Draw 16 X 15 cells

    # Draw vertical lines.
    HPDF_Page_SetLineWidth (page, 0.5)

    for i in range(18):
        x = i * CELL_WIDTH + 40

        HPDF_Page_MoveTo (page, x, PAGE_HEIGHT - 60)
        HPDF_Page_LineTo (page, x, 40)
        HPDF_Page_Stroke (page)

        if (i > 0 and i <= 16):
            HPDF_Page_BeginText (page)
            HPDF_Page_MoveTextPos (page, x + 5, PAGE_HEIGHT - 75)
            buf="%X" %(i - 1)
            HPDF_Page_ShowText (page, buf)
            HPDF_Page_EndText (page)

    # Draw horizontal lines.
    for i in range(16):
        y = i * CELL_HEIGHT + 40

        HPDF_Page_MoveTo (page, 40, y)
        HPDF_Page_LineTo (page, PAGE_WIDTH - 40, y)
        HPDF_Page_Stroke (page)

        if (i < 14):
            HPDF_Page_BeginText (page)
            HPDF_Page_MoveTextPos (page, 45, y + 5)

            buf="%X" %( 15 - i)
            HPDF_Page_ShowText (page, buf)
            HPDF_Page_EndText (page)


def draw_fonts (page):
    HPDF_Page_BeginText (page)

    # Draw all character from 0x20 to 0xFF to the canvas.
    for i in range(1,17):
        for j in range(1,17):
            buf=[None, None]
            y = PAGE_HEIGHT - 55 - ((i - 1) * CELL_HEIGHT)
            x = j * CELL_WIDTH + 50

            buf[1] = 0x00

            buf[0] = (i - 1) * 16 + (j - 1)
            if (buf[0] >= 32):
                d  = x - HPDF_Page_TextWidth (page, buf) / 2
                HPDF_Page_TextOut (page, d, y, buf)


    HPDF_Page_EndText (page)



def main ():

    encodings=[
            "StandardEncoding",
            "MacRomanEncoding",
            "WinAnsiEncoding",
            "ISO8859-2",
            "ISO8859-3",
            "ISO8859-4",
            "ISO8859-5",
            "ISO8859-9",
            "ISO8859-10",
            "ISO8859-13",
            "ISO8859-14",
            "ISO8859-15",
            "ISO8859-16",
            "CP1250",
            "CP1251",
            "CP1252",
            "CP1254",
            "CP1257",
            "KOI8-R",
            "Symbol-Set",
            "ZapfDingbats-Set",
            NULL
    ]

    pdf = HPDF_NewEx (error_handler, NULL, NULL, 0, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    # set compression mode
    HPDF_SetCompressionMode (pdf, HPDF_COMP_ALL)

    # Set page mode to use outlines.
    HPDF_SetPageMode(pdf, HPDF_PAGE_MODE_USE_OUTLINE)

    # get default font
    font = HPDF_GetFont (pdf, "Helvetica", NULL)

    # load font object
    font_name = HPDF_LoadType1FontFromFile (pdf, "type1/a010013l.afm",
            "type1/a010013l.pfb")

    # create outline root.
    root = HPDF_CreateOutline (pdf, NULL, "Encoding list", NULL)
    HPDF_Outline_SetOpened (root, HPDF_TRUE)

    i=0
    while (encodings[i]):
        page = HPDF_AddPage (pdf)

        HPDF_Page_SetWidth (page, PAGE_WIDTH)
        HPDF_Page_SetHeight (page, PAGE_HEIGHT)

        outline = HPDF_CreateOutline (pdf, root, encodings[i], NULL)
        dst = HPDF_Page_CreateDestination (page)
        HPDF_Destination_SetXYZ(dst, 0, HPDF_Page_GetHeight(page), 1)
        # HPDF_Destination_SetFitB(dst);
        HPDF_Outline_SetDestination(outline, dst)

        HPDF_Page_SetFontAndSize (page, font, 15)
        draw_graph (page)

        HPDF_Page_BeginText (page)
        HPDF_Page_SetFontAndSize (page, font, 20)
        HPDF_Page_MoveTextPos (page, 40, PAGE_HEIGHT - 50)
        HPDF_Page_ShowText (page, encodings[i])
        HPDF_Page_ShowText (page, " Encoding")
        HPDF_Page_EndText (page)

        if encodings[i]=="Symbol-Set":
            font2 = HPDF_GetFont (pdf, "Symbol", NULL)
        elif encodings[i]=="ZapfDingbats-Set":
            font2 = HPDF_GetFont (pdf, "ZapfDingbats", NULL)
        else:
            font2 = HPDF_GetFont (pdf, font_name, encodings[i])

        HPDF_Page_SetFontAndSize (page, font2, 14)
        draw_fonts (page)

        i+=1


    # save the document to a file
    HPDF_SaveToFile (pdf, fname)

    # clean up
    HPDF_Free (pdf)

    return 0

main()


###
## * << Haru Free PDF Library 2.0.0 >> -- font_demo.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *

from math import *


global pdf

@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: error_no=%04X, detail_no=%u\n", error_no,
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)


font_list=[
    "Courier",
    "Courier-Bold",
    "Courier-Oblique",
    "Courier-BoldOblique",
    "Helvetica",
    "Helvetica-Bold",
    "Helvetica-Oblique",
    "Helvetica-BoldOblique",
    "Times-Roman",
    "Times-Bold",
    "Times-Italic",
    "Times-BoldItalic",
    "Symbol",
    "ZapfDingbats",
]

def main ():
    global pdf

    page_title = "Font Demo"

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    pdf = HPDF_New (error_handler, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1


    # Add a new page object.
    page = HPDF_AddPage (pdf)

    height = HPDF_Page_GetHeight (page)
    width = HPDF_Page_GetWidth (page)

    # Print the lines of the page.
    HPDF_Page_SetLineWidth (page, 1)
    HPDF_Page_Rectangle (page, 50, 50, width - 100, height - 110)
    HPDF_Page_Stroke (page)

    # Print the title of the page (with positioning center).
    def_font = HPDF_GetFont (pdf, "Helvetica", NULL)
    HPDF_Page_SetFontAndSize (page, def_font, 24)

    tw = HPDF_Page_TextWidth (page, page_title)
    HPDF_Page_BeginText (page)
    HPDF_Page_TextOut (page, (width - tw) / 2, height - 50, page_title)
    HPDF_Page_EndText (page)

    # output subtitle.
    HPDF_Page_BeginText (page)
    HPDF_Page_SetFontAndSize (page, def_font, 16)
    HPDF_Page_TextOut (page, 60, height - 80, "<Standerd Type1 fonts samples>")
    HPDF_Page_EndText (page)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, 60, height - 105)

    for i in font_list:
        samp_text = "abcdefgABCDEFG12345!#$%&+-@?"

        #font_name = HPDF_LoadTTFontFromFile (pdf, "c:/winnt/fonts/arial.ttf", HPDF_TRUE);

        #font_name='arial.ttf'
        #font = HPDF_GetFont (pdf, font_name, "CP1250");

        #font = HPDF_GetFont (pdf, "Helvetica", NULL)

        font = HPDF_GetFont (pdf, i, NULL)

        # print a label of text
        HPDF_Page_SetFontAndSize (page, def_font, 9)
        HPDF_Page_ShowText (page, i)
        HPDF_Page_MoveTextPos (page, 0, -18)

        # print a sample text.
        HPDF_Page_SetFontAndSize (page, font, 20)
        HPDF_Page_ShowText (page, samp_text)
        HPDF_Page_MoveTextPos (page, 0, -20)


    HPDF_Page_EndText (page)

    HPDF_SaveToFile (pdf, fname)

    # clean up
    HPDF_Free (pdf)

    return 0

main()


###
## * << Haru Free PDF Library 2.0.0 >> -- permission.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *
from haru.hpdf_errorcode import *

text = "User cannot print and copy this document."
owner_passwd = "owner"
user_passwd = ""

@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: %s, detail_no=%u\n", error_detail[error_no],
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)


def main ():
    global  pdf

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    pdf = HPDF_New (error_handler, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1

    # create default-font
    font = HPDF_GetFont (pdf, "Helvetica", NULL)

    # add a new page object.
    page = HPDF_AddPage (pdf)

    HPDF_Page_SetSize (page, HPDF_PAGE_SIZE_B5, HPDF_PAGE_LANDSCAPE)

    HPDF_Page_BeginText (page)
    HPDF_Page_SetFontAndSize (page, font, 20)
    tw = HPDF_Page_TextWidth (page, text)
    HPDF_Page_MoveTextPos (page, (HPDF_Page_GetWidth (page) - tw) / 2,
                (HPDF_Page_GetHeight (page)  - 20) / 2)
    HPDF_Page_ShowText (page, text)
    HPDF_Page_EndText (page)

    HPDF_SetPassword (pdf, owner_passwd, user_passwd)
    HPDF_SetPermission (pdf, HPDF_ENABLE_READ)
    HPDF_SetEncryptionMode (pdf, HPDF_ENCRYPT_R3, 16)

    # save the document to a file
    HPDF_SaveToFile (pdf, fname)

    # clean up
    HPDF_Free (pdf)

    return 0

main()


###
## * << Haru Free PDF Library 2.0.8 >> -- hpdf_consts.h
## *
## * URL http://libharu.org/
## *
## * Copyright (c) 1999-2006 Takeshi Kanno
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

from hpdf_types import *

#----------------------------------------------------------------------------

HPDF_TRUE                  =1
HPDF_FALSE                 =0

HPDF_OK                    =0
HPDF_NOERROR               =0

#----- default values -------------------------------------------------------

# buffer size which is required when we convert to character string.
HPDF_TMP_BUF_SIZ           =512
HPDF_SHORT_BUF_SIZ         =32
HPDF_REAL_LEN              =11
HPDF_INT_LEN               =11
HPDF_TEXT_DEFAULT_LEN      =256
HPDF_UNICODE_HEADER_LEN    =2
HPDF_DATE_TIME_STR_LEN     =23

# length of each item defined in PDF
HPDF_BYTE_OFFSET_LEN       =10
HPDF_OBJ_ID_LEN            =7
HPDF_GEN_NO_LEN            =5

# default value of Graphic State
HPDF_DEF_FONT              ="Helvetica"
HPDF_DEF_PAGE_LAYOUT       =HPDF_PAGE_LAYOUT_SINGLE
HPDF_DEF_PAGE_MODE         =HPDF_PAGE_MODE_USE_NONE
HPDF_DEF_WORDSPACE         =0
HPDF_DEF_CHARSPACE         =0
HPDF_DEF_FONTSIZE          =10
HPDF_DEF_HSCALING          =100
HPDF_DEF_LEADING           =0
HPDF_DEF_RENDERING_MODE    =HPDF_FILL
HPDF_DEF_RISE              =0
HPDF_DEF_RAISE             =HPDF_DEF_RISE
HPDF_DEF_LINEWIDTH         =1
HPDF_DEF_LINECAP           =HPDF_BUTT_END
HPDF_DEF_LINEJOIN          =HPDF_MITER_JOIN
HPDF_DEF_MITERLIMIT        =10
HPDF_DEF_FLATNESS          =1
HPDF_DEF_PAGE_NUM          =1

HPDF_BS_DEF_WIDTH          =1

# default page-size
HPDF_DEF_PAGE_WIDTH        =595.276
HPDF_DEF_PAGE_HEIGHT       =841.89

HPDF_VERSION_TEXT          ="2.0.8"

#---------------------------------------------------------------------------
#----- compression mode ----------------------------------------------------

HPDF_COMP_NONE           =0x00
HPDF_COMP_TEXT           =0x01
HPDF_COMP_IMAGE          =0x02
HPDF_COMP_METADATA       =0x04
HPDF_COMP_ALL            =0x0F
HPDF_COMP_BEST_COMPRESS  =0x10
HPDF_COMP_BEST_SPEED     =0x20

HPDF_COMP_MASK           =0xFF


#----------------------------------------------------------------------------
#----- permission flags (only Revision 2 is supported)-----------------------

HPDF_ENABLE_READ        =0
HPDF_ENABLE_PRINT       =4
HPDF_ENABLE_EDIT_ALL    =8
HPDF_ENABLE_COPY        =16
HPDF_ENABLE_EDIT        =32


#----------------------------------------------------------------------------
#------ viewer preferences definitions --------------------------------------

HPDF_HIDE_TOOLBAR   =1
HPDF_HIDE_MENUBAR   =2
HPDF_HIDE_WINDOW_UI =4
HPDF_FIT_WINDOW     =8
HPDF_CENTER_WINDOW  =16


#---------------------------------------------------------------------------
#------ limitation of object implementation (PDF1.4) -----------------------

HPDF_LIMIT_MAX_INT            =2147483647
HPDF_LIMIT_MIN_INT            =-2147483647

HPDF_LIMIT_MAX_REAL           =32767
HPDF_LIMIT_MIN_REAL           =-32767

HPDF_LIMIT_MAX_STRING_LEN     =65535
HPDF_LIMIT_MAX_NAME_LEN       =127

HPDF_LIMIT_MAX_ARRAY          =8191
HPDF_LIMIT_MAX_DICT_ELEMENT   =4095
HPDF_LIMIT_MAX_XREF_ELEMENT   =8388607
HPDF_LIMIT_MAX_GSTATE         =28
HPDF_LIMIT_MAX_DEVICE_N       =8
HPDF_LIMIT_MAX_DEVICE_N_V15   =32
HPDF_LIMIT_MAX_CID            =65535
HPDF_MAX_GENERATION_NUM       =65535

HPDF_MIN_PAGE_HEIGHT          =3
HPDF_MIN_PAGE_WIDTH           =3
HPDF_MAX_PAGE_HEIGHT          =14400
HPDF_MAX_PAGE_WIDTH           =14400
HPDF_MIN_MAGNIFICATION_FACTOR =8
HPDF_MAX_MAGNIFICATION_FACTOR =3200

#---------------------------------------------------------------------------
#------ limitation of various properties -----------------------------------

HPDF_MIN_PAGE_SIZE         =3
HPDF_MAX_PAGE_SIZE         =14400
HPDF_MIN_HORIZONTALSCALING =10
HPDF_MAX_HORIZONTALSCALING =300
HPDF_MIN_WORDSPACE         =-30
HPDF_MAX_WORDSPACE         =300
HPDF_MIN_CHARSPACE         =-30
HPDF_MAX_CHARSPACE         =300
HPDF_MAX_FONTSIZE          =300
HPDF_MAX_ZOOMSIZE          =10
HPDF_MAX_LEADING           =300
HPDF_MAX_LINEWIDTH         =100
HPDF_MAX_DASH_PATTERN      =100

HPDF_MAX_JWW_NUM           =128

#----------------------------------------------------------------------------
#----- country code definition ----------------------------------------------

HPDF_COUNTRY_AF ="AF"    # AFGHANISTAN
HPDF_COUNTRY_AL ="AL"    # ALBANIA
HPDF_COUNTRY_DZ ="DZ"    # ALGERIA
HPDF_COUNTRY_AS ="AS"    # AMERICAN SAMOA
HPDF_COUNTRY_AD ="AD"    # ANDORRA
HPDF_COUNTRY_AO ="AO"    # ANGOLA
HPDF_COUNTRY_AI ="AI"    # ANGUILLA
HPDF_COUNTRY_AQ ="AQ"    # ANTARCTICA
HPDF_COUNTRY_AG ="AG"    # ANTIGUA AND BARBUDA
HPDF_COUNTRY_AR ="AR"    # ARGENTINA
HPDF_COUNTRY_AM ="AM"    # ARMENIA
HPDF_COUNTRY_AW ="AW"    # ARUBA
HPDF_COUNTRY_AU ="AU"    # AUSTRALIA
HPDF_COUNTRY_AT ="AT"    # AUSTRIA
HPDF_COUNTRY_AZ ="AZ"    # AZERBAIJAN
HPDF_COUNTRY_BS ="BS"    # BAHAMAS
HPDF_COUNTRY_BH ="BH"    # BAHRAIN
HPDF_COUNTRY_BD ="BD"    # BANGLADESH
HPDF_COUNTRY_BB ="BB"    # BARBADOS
HPDF_COUNTRY_BY ="BY"    # BELARUS
HPDF_COUNTRY_BE ="BE"    # BELGIUM
HPDF_COUNTRY_BZ ="BZ"    # BELIZE
HPDF_COUNTRY_BJ ="BJ"    # BENIN
HPDF_COUNTRY_BM ="BM"    # BERMUDA
HPDF_COUNTRY_BT ="BT"    # BHUTAN
HPDF_COUNTRY_BO ="BO"    # BOLIVIA
HPDF_COUNTRY_BA ="BA"    # BOSNIA AND HERZEGOWINA
HPDF_COUNTRY_BW ="BW"    # BOTSWANA
HPDF_COUNTRY_BV ="BV"    # BOUVET ISLAND
HPDF_COUNTRY_BR ="BR"    # BRAZIL
HPDF_COUNTRY_IO ="IO"    # BRITISH INDIAN OCEAN TERRITORY
HPDF_COUNTRY_BN ="BN"    # BRUNEI DARUSSALAM
HPDF_COUNTRY_BG ="BG"    # BULGARIA
HPDF_COUNTRY_BF ="BF"    # BURKINA FASO
HPDF_COUNTRY_BI ="BI"    # BURUNDI
HPDF_COUNTRY_KH ="KH"    # CAMBODIA
HPDF_COUNTRY_CM ="CM"    # CAMEROON
HPDF_COUNTRY_CA ="CA"    # CANADA
HPDF_COUNTRY_CV ="CV"    # CAPE VERDE
HPDF_COUNTRY_KY ="KY"    # CAYMAN ISLANDS
HPDF_COUNTRY_CF ="CF"    # CENTRAL AFRICAN REPUBLIC
HPDF_COUNTRY_TD ="TD"    # CHAD
HPDF_COUNTRY_CL ="CL"    # CHILE
HPDF_COUNTRY_CN ="CN"    # CHINA
HPDF_COUNTRY_CX ="CX"    # CHRISTMAS ISLAND
HPDF_COUNTRY_CC ="CC"    # COCOS (KEELING) ISLANDS
HPDF_COUNTRY_CO ="CO"    # COLOMBIA
HPDF_COUNTRY_KM ="KM"    # COMOROS
HPDF_COUNTRY_CG ="CG"    # CONGO
HPDF_COUNTRY_CK ="CK"    # COOK ISLANDS
HPDF_COUNTRY_CR ="CR"    # COSTA RICA
HPDF_COUNTRY_CI ="CI"    # COTE D'IVOIRE
HPDF_COUNTRY_HR ="HR"    # CROATIA (local name: Hrvatska)
HPDF_COUNTRY_CU ="CU"    # CUBA
HPDF_COUNTRY_CY ="CY"    # CYPRUS
HPDF_COUNTRY_CZ ="CZ"    # CZECH REPUBLIC
HPDF_COUNTRY_DK ="DK"    # DENMARK
HPDF_COUNTRY_DJ ="DJ"    # DJIBOUTI
HPDF_COUNTRY_DM ="DM"    # DOMINICA
HPDF_COUNTRY_DO ="DO"    # DOMINICAN REPUBLIC
HPDF_COUNTRY_TP ="TP"    # EAST TIMOR
HPDF_COUNTRY_EC ="EC"    # ECUADOR
HPDF_COUNTRY_EG ="EG"    # EGYPT
HPDF_COUNTRY_SV ="SV"    # EL SALVADOR
HPDF_COUNTRY_GQ ="GQ"    # EQUATORIAL GUINEA
HPDF_COUNTRY_ER ="ER"    # ERITREA
HPDF_COUNTRY_EE ="EE"    # ESTONIA
HPDF_COUNTRY_ET ="ET"    # ETHIOPIA
HPDF_COUNTRY_FK ="FK"   # FALKLAND ISLANDS (MALVINAS)
HPDF_COUNTRY_FO ="FO"    # FAROE ISLANDS
HPDF_COUNTRY_FJ ="FJ"    # FIJI
HPDF_COUNTRY_FI ="FI"    # FINLAND
HPDF_COUNTRY_FR ="FR"    # FRANCE
HPDF_COUNTRY_FX ="FX"    # FRANCE, METROPOLITAN
HPDF_COUNTRY_GF ="GF"    # FRENCH GUIANA
HPDF_COUNTRY_PF ="PF"    # FRENCH POLYNESIA
HPDF_COUNTRY_TF ="TF"    # FRENCH SOUTHERN TERRITORIES
HPDF_COUNTRY_GA ="GA"    # GABON
HPDF_COUNTRY_GM ="GM"    # GAMBIA
HPDF_COUNTRY_GE ="GE"    # GEORGIA
HPDF_COUNTRY_DE ="DE"    # GERMANY
HPDF_COUNTRY_GH ="GH"    # GHANA
HPDF_COUNTRY_GI ="GI"    # GIBRALTAR
HPDF_COUNTRY_GR ="GR"    # GREECE
HPDF_COUNTRY_GL ="GL"    # GREENLAND
HPDF_COUNTRY_GD ="GD"    # GRENADA
HPDF_COUNTRY_GP ="GP"    # GUADELOUPE
HPDF_COUNTRY_GU ="GU"    # GUAM
HPDF_COUNTRY_GT ="GT"    # GUATEMALA
HPDF_COUNTRY_GN ="GN"    # GUINEA
HPDF_COUNTRY_GW ="GW"    # GUINEA-BISSAU
HPDF_COUNTRY_GY ="GY"    # GUYANA
HPDF_COUNTRY_HT ="HT"    # HAITI
HPDF_COUNTRY_HM ="HM"    # HEARD AND MC DONALD ISLANDS
HPDF_COUNTRY_HN ="HN"    # HONDURAS
HPDF_COUNTRY_HK ="HK"    # HONG KONG
HPDF_COUNTRY_HU ="HU"    # HUNGARY
HPDF_COUNTRY_IS ="IS"    # ICELAND
HPDF_COUNTRY_IN ="IN"    # INDIA
HPDF_COUNTRY_ID ="ID"    # INDONESIA
HPDF_COUNTRY_IR ="IR"    # IRAN (ISLAMIC REPUBLIC OF)
HPDF_COUNTRY_IQ ="IQ"    # IRAQ
HPDF_COUNTRY_IE ="IE"    # IRELAND
HPDF_COUNTRY_IL ="IL"    # ISRAEL
HPDF_COUNTRY_IT ="IT"    # ITALY
HPDF_COUNTRY_JM ="JM"    # JAMAICA
HPDF_COUNTRY_JP ="JP"    # JAPAN
HPDF_COUNTRY_JO ="JO"    # JORDAN
HPDF_COUNTRY_KZ ="KZ"    # KAZAKHSTAN
HPDF_COUNTRY_KE ="KE"    # KENYA
HPDF_COUNTRY_KI ="KI"    # KIRIBATI
HPDF_COUNTRY_KP ="KP"    # KOREA, DEMOCRATIC PEOPLE'S REPUBLIC OF
HPDF_COUNTRY_KR ="KR"    # KOREA, REPUBLIC OF
HPDF_COUNTRY_KW ="KW"    # KUWAIT
HPDF_COUNTRY_KG ="KG"    # KYRGYZSTAN
HPDF_COUNTRY_LA ="LA"    # LAO PEOPLE'S DEMOCRATIC REPUBLIC
HPDF_COUNTRY_LV ="LV"    # LATVIA
HPDF_COUNTRY_LB ="LB"    # LEBANON
HPDF_COUNTRY_LS ="LS"    # LESOTHO
HPDF_COUNTRY_LR ="LR"    # LIBERIA
HPDF_COUNTRY_LY ="LY"    # LIBYAN ARAB JAMAHIRIYA
HPDF_COUNTRY_LI ="LI"    # LIECHTENSTEIN
HPDF_COUNTRY_LT ="LT"    # LITHUANIA
HPDF_COUNTRY_LU ="LU"    # LUXEMBOURG
HPDF_COUNTRY_MO ="MO"    # MACAU
HPDF_COUNTRY_MK ="MK"    # MACEDONIA, THE FORMER YUGOSLAV REPUBLIC OF
HPDF_COUNTRY_MG ="MG"    # MADAGASCAR
HPDF_COUNTRY_MW ="MW"    # MALAWI
HPDF_COUNTRY_MY ="MY"    # MALAYSIA
HPDF_COUNTRY_MV ="MV"    # MALDIVES
HPDF_COUNTRY_ML ="ML"    # MALI
HPDF_COUNTRY_MT ="MT"    # MALTA
HPDF_COUNTRY_MH ="MH"    # MARSHALL ISLANDS
HPDF_COUNTRY_MQ ="MQ"    # MARTINIQUE
HPDF_COUNTRY_MR ="MR"    # MAURITANIA
HPDF_COUNTRY_MU ="MU"    # MAURITIUS
HPDF_COUNTRY_YT ="YT"    # MAYOTTE
HPDF_COUNTRY_MX ="MX"    # MEXICO
HPDF_COUNTRY_FM ="FM"    # MICRONESIA, FEDERATED STATES OF
HPDF_COUNTRY_MD ="MD"    # MOLDOVA, REPUBLIC OF
HPDF_COUNTRY_MC ="MC"    # MONACO
HPDF_COUNTRY_MN ="MN"    # MONGOLIA
HPDF_COUNTRY_MS ="MS"    # MONTSERRAT
HPDF_COUNTRY_MA ="MA"    # MOROCCO
HPDF_COUNTRY_MZ ="MZ"    # MOZAMBIQUE
HPDF_COUNTRY_MM ="MM"    # MYANMAR
HPDF_COUNTRY_NA ="NA"    # NAMIBIA
HPDF_COUNTRY_NR ="NR"    # NAURU
HPDF_COUNTRY_NP ="NP"    # NEPAL
HPDF_COUNTRY_NL ="NL"    # NETHERLANDS
HPDF_COUNTRY_AN ="AN"    # NETHERLANDS ANTILLES
HPDF_COUNTRY_NC ="NC"    # NEW CALEDONIA
HPDF_COUNTRY_NZ ="NZ"    # NEW ZEALAND
HPDF_COUNTRY_NI ="NI"    # NICARAGUA
HPDF_COUNTRY_NE ="NE"    # NIGER
HPDF_COUNTRY_NG ="NG"    # NIGERIA
HPDF_COUNTRY_NU ="NU"    # NIUE
HPDF_COUNTRY_NF ="NF"    # NORFOLK ISLAND
HPDF_COUNTRY_MP ="MP"    # NORTHERN MARIANA ISLANDS
HPDF_COUNTRY_NO ="NO"    # NORWAY
HPDF_COUNTRY_OM ="OM"    # OMAN
HPDF_COUNTRY_PK ="PK"    # PAKISTAN
HPDF_COUNTRY_PW ="PW"    # PALAU
HPDF_COUNTRY_PA ="PA"    # PANAMA
HPDF_COUNTRY_PG ="PG"    # PAPUA NEW GUINEA
HPDF_COUNTRY_PY ="PY"    # PARAGUAY
HPDF_COUNTRY_PE ="PE"    # PERU
HPDF_COUNTRY_PH ="PH"    # PHILIPPINES
HPDF_COUNTRY_PN ="PN"    # PITCAIRN
HPDF_COUNTRY_PL ="PL"    # POLAND
HPDF_COUNTRY_PT ="PT"    # PORTUGAL
HPDF_COUNTRY_PR ="PR"    # PUERTO RICO
HPDF_COUNTRY_QA ="QA"    # QATAR
HPDF_COUNTRY_RE ="RE"    # REUNION
HPDF_COUNTRY_RO ="RO"    # ROMANIA
HPDF_COUNTRY_RU ="RU"    # RUSSIAN FEDERATION
HPDF_COUNTRY_RW ="RW"    # RWANDA
HPDF_COUNTRY_KN ="KN"    # SAINT KITTS AND NEVIS
HPDF_COUNTRY_LC ="LC"    # SAINT LUCIA
HPDF_COUNTRY_VC ="VC"    # SAINT VINCENT AND THE GRENADINES
HPDF_COUNTRY_WS ="WS"    # SAMOA
HPDF_COUNTRY_SM ="SM"    # SAN MARINO
HPDF_COUNTRY_ST ="ST"    # SAO TOME AND PRINCIPE
HPDF_COUNTRY_SA ="SA"    # SAUDI ARABIA
HPDF_COUNTRY_SN ="SN"    # SENEGAL
HPDF_COUNTRY_SC ="SC"    # SEYCHELLES
HPDF_COUNTRY_SL ="SL"    # SIERRA LEONE
HPDF_COUNTRY_SG ="SG"    # SINGAPORE
HPDF_COUNTRY_SK ="SK"    # SLOVAKIA (Slovak Republic)
HPDF_COUNTRY_SI ="SI"    # SLOVENIA
HPDF_COUNTRY_SB ="SB"    # SOLOMON ISLANDS
HPDF_COUNTRY_SO ="SO"    # SOMALIA
HPDF_COUNTRY_ZA ="ZA"    # SOUTH AFRICA
HPDF_COUNTRY_ES ="ES"    # SPAIN
HPDF_COUNTRY_LK ="LK"    # SRI LANKA
HPDF_COUNTRY_SH ="SH"    # ST. HELENA
HPDF_COUNTRY_PM ="PM"    # ST. PIERRE AND MIQUELON
HPDF_COUNTRY_SD ="SD"    # SUDAN
HPDF_COUNTRY_SR ="SR"    # SURINAME
HPDF_COUNTRY_SJ ="SJ"    # SVALBARD AND JAN MAYEN ISLANDS
HPDF_COUNTRY_SZ ="SZ"    # SWAZILAND
HPDF_COUNTRY_SE ="SE"    # SWEDEN
HPDF_COUNTRY_CH ="CH"    # SWITZERLAND
HPDF_COUNTRY_SY ="SY"    # SYRIAN ARAB REPUBLIC
HPDF_COUNTRY_TW ="TW"    # TAIWAN, PROVINCE OF CHINA
HPDF_COUNTRY_TJ ="TJ"    # TAJIKISTAN
HPDF_COUNTRY_TZ ="TZ"    # TANZANIA, UNITED REPUBLIC OF
HPDF_COUNTRY_TH ="TH"    # THAILAND
HPDF_COUNTRY_TG ="TG"    # TOGO
HPDF_COUNTRY_TK ="TK"    # TOKELAU
HPDF_COUNTRY_TO ="TO"    # TONGA
HPDF_COUNTRY_TT ="TT"    # TRINIDAD AND TOBAGO
HPDF_COUNTRY_TN ="TN"    # TUNISIA
HPDF_COUNTRY_TR ="TR"    # TURKEY
HPDF_COUNTRY_TM ="TM"    # TURKMENISTAN
HPDF_COUNTRY_TC ="TC"    # TURKS AND CAICOS ISLANDS
HPDF_COUNTRY_TV ="TV"    # TUVALU
HPDF_COUNTRY_UG ="UG"    # UGANDA
HPDF_COUNTRY_UA ="UA"    # UKRAINE
HPDF_COUNTRY_AE ="AE"    # UNITED ARAB EMIRATES
HPDF_COUNTRY_GB ="GB"    # UNITED KINGDOM
HPDF_COUNTRY_US ="US"    # UNITED STATES
HPDF_COUNTRY_UM ="UM"    # UNITED STATES MINOR OUTLYING ISLANDS
HPDF_COUNTRY_UY ="UY"    # URUGUAY
HPDF_COUNTRY_UZ ="UZ"    # UZBEKISTAN
HPDF_COUNTRY_VU ="VU"    # VANUATU
HPDF_COUNTRY_VA ="VA"    # VATICAN CITY STATE (HOLY SEE)
HPDF_COUNTRY_VE ="VE"    # VENEZUELA
HPDF_COUNTRY_VN ="VN"    # VIET NAM
HPDF_COUNTRY_VG ="VG"    # VIRGIN ISLANDS (BRITISH)
HPDF_COUNTRY_VI ="VI"    # VIRGIN ISLANDS (U.S.)
HPDF_COUNTRY_WF ="WF"    # WALLIS AND FUTUNA ISLANDS
HPDF_COUNTRY_EH ="EH"    # WESTERN SAHARA
HPDF_COUNTRY_YE ="YE"    # YEMEN
HPDF_COUNTRY_YU ="YU"    # YUGOSLAVIA
HPDF_COUNTRY_ZR ="ZR"    # ZAIRE
HPDF_COUNTRY_ZM ="ZM"    # ZAMBIA
HPDF_COUNTRY_ZW ="ZW"    # ZIMBABWE

#----------------------------------------------------------------------------
#----- lang code definition -------------------------------------------------

HPDF_LANG_AA   ="aa"     # Afar
HPDF_LANG_AB   ="ab"     # Abkhazian
HPDF_LANG_AF   ="af"     # Afrikaans
HPDF_LANG_AM   ="am"     # Amharic
HPDF_LANG_AR   ="ar"     # Arabic
HPDF_LANG_AS   ="as"     # Assamese
HPDF_LANG_AY   ="ay"     # Aymara
HPDF_LANG_AZ   ="az"     # Azerbaijani
HPDF_LANG_BA   ="ba"     # Bashkir
HPDF_LANG_BE   ="be"     # Byelorussian
HPDF_LANG_BG   ="bg"     # Bulgarian
HPDF_LANG_BH   ="bh"     # Bihari
HPDF_LANG_BI   ="bi"     # Bislama
HPDF_LANG_BN   ="bn"     # Bengali Bangla
HPDF_LANG_BO   ="bo"     # Tibetan
HPDF_LANG_BR   ="br"     # Breton
HPDF_LANG_CA   ="ca"     # Catalan
HPDF_LANG_CO   ="co"     # Corsican
HPDF_LANG_CS   ="cs"     # Czech
HPDF_LANG_CY   ="cy"     # Welsh
HPDF_LANG_DA   ="da"     # Danish
HPDF_LANG_DE   ="de"     # German
HPDF_LANG_DZ   ="dz"     # Bhutani
HPDF_LANG_EL   ="el"     # Greek
HPDF_LANG_EN   ="en"     # English
HPDF_LANG_EO   ="eo"     # Esperanto
HPDF_LANG_ES   ="es"     # Spanish
HPDF_LANG_ET   ="et"     # Estonian
HPDF_LANG_EU   ="eu"     # Basque
HPDF_LANG_FA   ="fa"     # Persian
HPDF_LANG_FI   ="fi"     # Finnish
HPDF_LANG_FJ   ="fj"     # Fiji
HPDF_LANG_FO   ="fo"     # Faeroese
HPDF_LANG_FR   ="fr"     # French
HPDF_LANG_FY   ="fy"     # Frisian
HPDF_LANG_GA   ="ga"     # Irish
HPDF_LANG_GD   ="gd"     # Scots Gaelic
HPDF_LANG_GL   ="gl"     # Galician
HPDF_LANG_GN   ="gn"     # Guarani
HPDF_LANG_GU   ="gu"     # Gujarati
HPDF_LANG_HA   ="ha"     # Hausa
HPDF_LANG_HI   ="hi"     # Hindi
HPDF_LANG_HR   ="hr"     # Croatian
HPDF_LANG_HU   ="hu"     # Hungarian
HPDF_LANG_HY   ="hy"     # Armenian
HPDF_LANG_IA   ="ia"     # Interlingua
HPDF_LANG_IE   ="ie"     # Interlingue
HPDF_LANG_IK   ="ik"     # Inupiak
HPDF_LANG_IN   ="in"     # Indonesian
HPDF_LANG_IS   ="is"     # Icelandic
HPDF_LANG_IT   ="it"     # Italian
HPDF_LANG_IW   ="iw"     # Hebrew
HPDF_LANG_JA   ="ja"     # Japanese
HPDF_LANG_JI   ="ji"     # Yiddish
HPDF_LANG_JW   ="jw"     # Javanese
HPDF_LANG_KA   ="ka"     # Georgian
HPDF_LANG_KK   ="kk"     # Kazakh
HPDF_LANG_KL   ="kl"     # Greenlandic
HPDF_LANG_KM   ="km"     # Cambodian
HPDF_LANG_KN   ="kn"     # Kannada
HPDF_LANG_KO   ="ko"     # Korean
HPDF_LANG_KS   ="ks"     # Kashmiri
HPDF_LANG_KU   ="ku"     # Kurdish
HPDF_LANG_KY   ="ky"     # Kirghiz
HPDF_LANG_LA   ="la"     # Latin
HPDF_LANG_LN   ="ln"     # Lingala
HPDF_LANG_LO   ="lo"     # Laothian
HPDF_LANG_LT   ="lt"     # Lithuanian
HPDF_LANG_LV   ="lv"     # Latvian,Lettish
HPDF_LANG_MG   ="mg"     # Malagasy
HPDF_LANG_MI   ="mi"     # Maori
HPDF_LANG_MK   ="mk"     # Macedonian
HPDF_LANG_ML   ="ml"     # Malayalam
HPDF_LANG_MN   ="mn"     # Mongolian
HPDF_LANG_MO   ="mo"     # Moldavian
HPDF_LANG_MR   ="mr"     # Marathi
HPDF_LANG_MS   ="ms"     # Malay
HPDF_LANG_MT   ="mt"     # Maltese
HPDF_LANG_MY   ="my"     # Burmese
HPDF_LANG_NA   ="na"     # Nauru
HPDF_LANG_NE   ="ne"     # Nepali
HPDF_LANG_NL   ="nl"     # Dutch
HPDF_LANG_NO   ="no"     # Norwegian
HPDF_LANG_OC   ="oc"     # Occitan
HPDF_LANG_OM   ="om"     # (Afan)Oromo
HPDF_LANG_OR   ="or"     # Oriya
HPDF_LANG_PA   ="pa"     # Punjabi
HPDF_LANG_PL   ="pl"     # Polish
HPDF_LANG_PS   ="ps"     # Pashto,Pushto
HPDF_LANG_PT   ="pt"     # Portuguese
HPDF_LANG_QU   ="qu"     # Quechua
HPDF_LANG_RM   ="rm"     # Rhaeto-Romance
HPDF_LANG_RN   ="rn"     # Kirundi
HPDF_LANG_RO   ="ro"     # Romanian
HPDF_LANG_RU   ="ru"     # Russian
HPDF_LANG_RW   ="rw"     # Kinyarwanda
HPDF_LANG_SA   ="sa"     # Sanskrit
HPDF_LANG_SD   ="sd"     # Sindhi
HPDF_LANG_SG   ="sg"     # Sangro
HPDF_LANG_SH   ="sh"     # Serbo-Croatian
HPDF_LANG_SI   ="si"     # Singhalese
HPDF_LANG_SK   ="sk"     # Slovak
HPDF_LANG_SL   ="sl"     # Slovenian
HPDF_LANG_SM   ="sm"     # Samoan
HPDF_LANG_SN   ="sn"     # Shona
HPDF_LANG_SO   ="so"     # Somali
HPDF_LANG_SQ   ="sq"     # Albanian
HPDF_LANG_SR   ="sr"     # Serbian
HPDF_LANG_SS   ="ss"     # Siswati
HPDF_LANG_ST   ="st"     # Sesotho
HPDF_LANG_SU   ="su"     # Sundanese
HPDF_LANG_SV   ="sv"     # Swedish
HPDF_LANG_SW   ="sw"     # Swahili
HPDF_LANG_TA   ="ta"     # Tamil
HPDF_LANG_TE   ="te"     # Tegulu
HPDF_LANG_TG   ="tg"     # Tajik
HPDF_LANG_TH   ="th"     # Thai
HPDF_LANG_TI   ="ti"     # Tigrinya
HPDF_LANG_TK   ="tk"     # Turkmen
HPDF_LANG_TL   ="tl"     # Tagalog
HPDF_LANG_TN   ="tn"     # Setswanato Tonga
HPDF_LANG_TR   ="tr"     # Turkish
HPDF_LANG_TS   ="ts"     # Tsonga
HPDF_LANG_TT   ="tt"     # Tatar
HPDF_LANG_TW   ="tw"     # Twi
HPDF_LANG_UK   ="uk"     # Ukrainian
HPDF_LANG_UR   ="ur"     # Urdu
HPDF_LANG_UZ   ="uz"     # Uzbek
HPDF_LANG_VI   ="vi"     # Vietnamese
HPDF_LANG_VO   ="vo"     # Volapuk
HPDF_LANG_WO   ="wo"     # Wolof
HPDF_LANG_XH   ="xh"     # Xhosa
HPDF_LANG_YO   ="yo"     # Yoruba
HPDF_LANG_ZH   ="zh"     # Chinese
HPDF_LANG_ZU   ="zu"     # Zulu


#----------------------------------------------------------------------------
#----- Graphics mode --------------------------------------------------------

HPDF_GMODE_PAGE_DESCRIPTION      =0x0001
HPDF_GMODE_PATH_OBJECT           =0x0002
HPDF_GMODE_TEXT_OBJECT           =0x0004
HPDF_GMODE_CLIPPING_PATH         =0x0008
HPDF_GMODE_SHADING               =0x0010
HPDF_GMODE_INLINE_IMAGE          =0x0020
HPDF_GMODE_EXTERNAL_OBJECT       =0x0040



###
## * << Haru Free PDF Library 2.0.3 >> -- hpdf_types.h
## *
## * URL http://libharu.org/
## *
## * Copyright (c) 1999-2006 Takeshi Kanno
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##
## port to python by Li Jun
## http://groups.google.com/group/pythoncia

from ctypes import *

#----------------------------------------------------------------------------
#----- type definition ------------------------------------------------------


#  native OS integer types
HPDF_INT=c_int
HPDF_UINT=c_uint


#  32bit integer types
HPDF_INT32=c_int
HPDF_UINT32=c_uint


#  16bit integer types
HPDF_INT16=c_short
HPDF_UINT16=c_ushort


#  8bit integer types
HPDF_INT8=c_char
HPDF_UINT8=c_ubyte


#  8bit binary types
HPDF_BYTE=c_ubyte


#  float type (32bit IEEE754)
HPDF_REAL=c_float


#  double type (64bit IEEE754)
HPDF_DOUBLE=c_double


#  boolean type (0: False, !0: True)
HPDF_BOOL=c_int


#  error-no type (32bit unsigned integer)
HPDF_STATUS=c_ulong


#  character-code type (16bit)
HPDF_CID=HPDF_UINT16
HPDF_UNICODE=HPDF_UINT16


#  HPDF_Point struct
class _HPDF_Point(Structure):
    _fields_=[
              ('x',            HPDF_REAL),
              ('y',            HPDF_REAL),
             ]
HPDF_Point = _HPDF_Point

class _HPDF_Rect(Structure):
    _fields_=[
              ('left',         HPDF_REAL),
              ('bottom',       HPDF_REAL),
              ('right',        HPDF_REAL),
              ('top',          HPDF_REAL),
             ]
HPDF_Rect = _HPDF_Rect

HPDF_Box= _HPDF_Rect

# HPDF_Date struct
class _HPDF_Date(Structure):
    _fields_=[
              ('year',         HPDF_INT),
              ('month',        HPDF_INT),
              ('day',          HPDF_INT),
              ('hour',         HPDF_INT),
              ('minutes',      HPDF_INT),
              ('seconds',      HPDF_INT),
              ('ind',          c_char),
              ('off_hour',     HPDF_INT),
              ('off_minutes',  HPDF_INT),
             ]
HPDF_Date = _HPDF_Date


#enum starts
## date-time type parameters
HPDF_INFO_CREATION_DATE     =0
HPDF_INFO_MOD_DATE          =1

## string type parameters
HPDF_INFO_AUTHOR            =2
HPDF_INFO_CREATOR           =3
HPDF_INFO_PRODUCER          =4
HPDF_INFO_TITLE             =5
HPDF_INFO_SUBJECT           =6
HPDF_INFO_KEYWORDS          =7
HPDF_INFO_EOF               =8
HPDF_InfoType               =c_int
#enum ends

#enum starts
HPDF_VER_12                 =0
HPDF_VER_13                 =1
HPDF_VER_14                 =2
HPDF_VER_15                 =3
HPDF_VER_16                 =4
HPDF_VER_EOF                =5
HPDF_PDFVer                 =c_int
#enum ends



#enum starts
HPDF_ENCRYPT_R2             =2
HPDF_ENCRYPT_R3             =3
HPDF_EncryptMode            =c_int
#enum ends




##typedef void
##(HPDF_STDCALL *HPDF_Error_Handler)  (HPDF_STATUS   error_no,
##                                     HPDF_STATUS   detail_no,
##                                     void         *user_data)
def HPDF_Error_Handler(restype,
                       error_no,
                       detail_no,
                       user_data):
    return CFUNCTYPE(restype,
                     error_no,
                     detail_no,
                     user_data,
                    )

#typedef void*
#(HPDF_STDCALL *HPDF_Alloc_Func)  (HPDF_UINT  size)
HPDF_Alloc_Func=CFUNCTYPE(c_void_p,
                          HPDF_UINT,        #size
                         )


#typedef void
#(HPDF_STDCALL *HPDF_Free_Func)  (void  *aptr)
HPDF_Free_Func=CFUNCTYPE(None,
                         c_void_p,          #aptr
                        )


#---------------------------------------------------------------------------
#------ text width struct --------------------------------------------------

class _HPDF_TextWidth(Structure):
    _fields_=[
              ('numchars',            HPDF_UINT),

              # don't use this value (it may be change in the feature).
              # use numspace as alternated.
              ('numwords',            HPDF_UINT),

              ('width',               HPDF_UINT),
              ('numspace',            HPDF_UINT),
             ]
HPDF_TextWidth = _HPDF_TextWidth


#---------------------------------------------------------------------------
#------ dash mode ----------------------------------------------------------

class _HPDF_DashMode(Structure):
    _fields_=[
              ('ptn',                 HPDF_UINT16*8),
              ('num_ptn',             HPDF_UINT),
              ('phase',               HPDF_UINT),
             ]
HPDF_DashMode = _HPDF_DashMode


#---------------------------------------------------------------------------
#----- HPDF_TransMatrix struct ---------------------------------------------

class _HPDF_TransMatrix(Structure):
    _fields_=[
              ('a',                   HPDF_REAL),
              ('b',                   HPDF_REAL),
              ('c',                   HPDF_REAL),
              ('d',                   HPDF_REAL),
              ('x',                   HPDF_REAL),
              ('y',                   HPDF_REAL),
             ]
HPDF_TransMatrix = _HPDF_TransMatrix




#---------------------------------------------------------------------------

#enum starts
HPDF_CS_DEVICE_GRAY            =0
HPDF_CS_DEVICE_RGB             =1
HPDF_CS_DEVICE_CMYK            =2
HPDF_CS_CAL_GRAY               =3
HPDF_CS_CAL_RGB                =4
HPDF_CS_LAB                    =5
HPDF_CS_ICC_BASED              =6
HPDF_CS_SEPARATION             =7
HPDF_CS_DEVICE_N               =8
HPDF_CS_INDEXED                =9
HPDF_CS_PATTERN                =10
HPDF_CS_EOF                    =11
HPDF_ColorSpace                =c_int
#enum ends



#---------------------------------------------------------------------------
#----- HPDF_RGBColor struct ------------------------------------------------

class _HPDF_RGBColor(Structure):
    _fields_=[
              ('r',            HPDF_REAL),
              ('g',            HPDF_REAL),
              ('b',            HPDF_REAL),
             ]
HPDF_RGBColor = _HPDF_RGBColor

#---------------------------------------------------------------------------
#----- HPDF_CMYKColor struct -----------------------------------------------

class _HPDF_CMYKColor(Structure):
    _fields_=[
              ('c',             HPDF_REAL),
              ('m',             HPDF_REAL),
              ('y',             HPDF_REAL),
              ('k',             HPDF_REAL),
             ]
HPDF_CMYKColor=_HPDF_CMYKColor

#---------------------------------------------------------------------------
#------ The line cap style -------------------------------------------------

#enum starts
HPDF_BUTT_END                                =0
HPDF_ROUND_END                               =1
HPDF_PROJECTING_SQUARE_END                   =2
HPDF_LINECAP_EOF                             =3
HPDF_LineCap                                 =c_int
#enum ends



#----------------------------------------------------------------------------
#------ The line join style -------------------------------------------------

#enum starts
HPDF_MITER_JOIN                              =0
HPDF_ROUND_JOIN                              =1
HPDF_BEVEL_JOIN                              =2
HPDF_LINEJOIN_EOF                            =3
HPDF_LineJoin                                =c_int
#enum ends



#----------------------------------------------------------------------------
#------ The text rendering mode ---------------------------------------------

#enum starts
HPDF_FILL                                    =0
HPDF_STROKE                                  =1
HPDF_FILL_THEN_STROKE                        =2
HPDF_INVISIBLE                               =3
HPDF_FILL_CLIPPING                           =4
HPDF_STROKE_CLIPPING                         =5
HPDF_FILL_STROKE_CLIPPING                    =6
HPDF_CLIPPING                                =7
HPDF_RENDERING_MODE_EOF                      =8
HPDF_TextRenderingMode                       =c_int
#enum ends




#enum starts
HPDF_WMODE_HORIZONTAL                        =0
HPDF_WMODE_VERTICAL                          =1
HPDF_WMODE_EOF                               =2
HPDF_WritingMode                             =c_int
#enum ends




#enum starts
HPDF_PAGE_LAYOUT_SINGLE                      =0
HPDF_PAGE_LAYOUT_ONE_COLUMN                  =1
HPDF_PAGE_LAYOUT_TWO_COLUMN_LEFT             =2
HPDF_PAGE_LAYOUT_TWO_COLUMN_RIGHT            =3
HPDF_PAGE_LAYOUT_EOF                         =4
HPDF_PageLayout                              =c_int
#enum ends




#enum starts
HPDF_PAGE_MODE_USE_NONE                      =0
HPDF_PAGE_MODE_USE_OUTLINE                   =1
HPDF_PAGE_MODE_USE_THUMBS                    =2
HPDF_PAGE_MODE_FULL_SCREEN                   =3
HPDF_PAGE_MODE_USE_OC                        =4     #???
HPDF_PAGE_MODE_USE_ATTACHMENTS               =5     #???
HPDF_PAGE_MODE_EOF                           =6
HPDF_PageMode                                =c_int
#enum ends




#enum starts
HPDF_PAGE_NUM_STYLE_DECIMAL                  =0
HPDF_PAGE_NUM_STYLE_UPPER_ROMAN              =1
HPDF_PAGE_NUM_STYLE_LOWER_ROMAN              =2
HPDF_PAGE_NUM_STYLE_UPPER_LETTERS            =3
HPDF_PAGE_NUM_STYLE_LOWER_LETTERS            =4
HPDF_PAGE_NUM_STYLE_EOF                      =5
HPDF_PageNumStyle                            =c_int
#enum ends




#enum starts
HPDF_XYZ                                     =0
HPDF_FIT                                     =1
HPDF_FIT_H                                   =2
HPDF_FIT_V                                   =3
HPDF_FIT_R                                   =4
HPDF_FIT_B                                   =5
HPDF_FIT_BH                                  =6
HPDF_FIT_BV                                  =7
HPDF_DST_EOF                                 =8
HPDF_DestinationType                         =c_int
#enum ends




#enum starts
HPDF_ANNOT_TEXT_NOTES                        =0
HPDF_ANNOT_LINK                              =1
HPDF_ANNOT_SOUND                             =2
HPDF_ANNOT_FREE_TEXT                         =3
HPDF_ANNOT_STAMP                             =4
HPDF_ANNOT_SQUARE                            =5
HPDF_ANNOT_CIRCLE                            =6
HPDF_ANNOT_STRIKE_OUT                        =7
HPDF_ANNOT_HIGHTLIGHT                        =8
HPDF_ANNOT_UNDERLINE                         =9
HPDF_ANNOT_INK                               =10
HPDF_ANNOT_FILE_ATTACHMENT                   =11
HPDF_ANNOT_POPUP                             =12
HPDF_AnnotType                               =c_int
#enum ends




#enum starts
HPDF_ANNOT_INVISIBLE                         =0
HPDF_ANNOT_HIDDEN                            =1
HPDF_ANNOT_PRINT                             =2
HPDF_ANNOT_NOZOOM                            =3
HPDF_ANNOT_NOROTATE                          =4
HPDF_ANNOT_NOVIEW                            =5
HPDF_ANNOT_READONLY                          =6
HPDF_AnnotFlgs                               =c_int
#enum ends




#enum starts
HPDF_ANNOT_NO_HIGHTLIGHT                     =0
HPDF_ANNOT_INVERT_BOX                        =1
HPDF_ANNOT_INVERT_BORDER                     =2
HPDF_ANNOT_DOWN_APPEARANCE                   =3
HPDF_ANNOT_HIGHTLIGHT_MODE_EOF               =4
HPDF_AnnotHighlightMode                      =c_int
#enum ends




#enum starts
HPDF_ANNOT_ICON_COMMENT                      =0
HPDF_ANNOT_ICON_KEY                          =1
HPDF_ANNOT_ICON_NOTE                         =2
HPDF_ANNOT_ICON_HELP                         =3
HPDF_ANNOT_ICON_NEW_PARAGRAPH                =4
HPDF_ANNOT_ICON_PARAGRAPH                    =5
HPDF_ANNOT_ICON_INSERT                       =6
HPDF_ANNOT_ICON_EOF                          =7
HPDF_AnnotIcon                               =c_int
#enum ends





#----------------------------------------------------------------------------
#------ border stype --------------------------------------------------------

#enum starts
HPDF_BS_SOLID                                =0
HPDF_BS_DASHED                               =1
HPDF_BS_BEVELED                              =2
HPDF_BS_INSET                                =3
HPDF_BS_UNDERLINED                           =4
HPDF_BSSubtype                               =c_int
#enum ends




#----- blend modes ----------------------------------------------------------

#enum starts
HPDF_BM_NORMAL                               =0
HPDF_BM_MULTIPLY                             =1
HPDF_BM_SCREEN                               =2
HPDF_BM_OVERLAY                              =3
HPDF_BM_DARKEN                               =4
HPDF_BM_LIGHTEN                              =5
HPDF_BM_COLOR_DODGE                          =6
HPDF_BM_COLOR_BUM                            =7
HPDF_BM_HARD_LIGHT                           =8
HPDF_BM_SOFT_LIGHT                           =9
HPDF_BM_DIFFERENCE                           =10
HPDF_BM_EXCLUSHON                            =11
HPDF_BM_EOF                                  =12
HPDF_BlendMode                               =c_int
#enum ends



#----- slide show -----------------------------------------------------------

#enum starts
HPDF_TS_WIPE_RIGHT                           =0
HPDF_TS_WIPE_UP                              =1
HPDF_TS_WIPE_LEFT                            =2
HPDF_TS_WIPE_DOWN                            =3
HPDF_TS_BARN_DOORS_HORIZONTAL_OUT            =4
HPDF_TS_BARN_DOORS_HORIZONTAL_IN             =5
HPDF_TS_BARN_DOORS_VERTICAL_OUT              =6
HPDF_TS_BARN_DOORS_VERTICAL_IN               =7
HPDF_TS_BOX_OUT                              =8
HPDF_TS_BOX_IN                               =9
HPDF_TS_BLINDS_HORIZONTAL                    =10
HPDF_TS_BLINDS_VERTICAL                      =11
HPDF_TS_DISSOLVE                             =12
HPDF_TS_GLITTER_RIGHT                        =13
HPDF_TS_GLITTER_DOWN                         =14
HPDF_TS_GLITTER_TOP_LEFT_TO_BOTTOM_RIGHT     =15
HPDF_TS_REPLACE                              =16
HPDF_TS_EOF                                  =17
HPDF_TransitionStyle                         =c_int
#enum ends



#----------------------------------------------------------------------------

#enum starts
HPDF_PAGE_SIZE_LETTER                        =0
HPDF_PAGE_SIZE_LEGAL                         =1
HPDF_PAGE_SIZE_A3                            =2
HPDF_PAGE_SIZE_A4                            =3
HPDF_PAGE_SIZE_A5                            =4
HPDF_PAGE_SIZE_B4                            =5
HPDF_PAGE_SIZE_B5                            =6
HPDF_PAGE_SIZE_EXECUTIVE                     =7
HPDF_PAGE_SIZE_US4x6                         =8
HPDF_PAGE_SIZE_US4x8                         =9
HPDF_PAGE_SIZE_US5x7                         =10
HPDF_PAGE_SIZE_COMM10                        =11
HPDF_PAGE_SIZE_EOF                           =12
HPDF_PageSizes                               =c_int
#enum ends




#enum starts
HPDF_PAGE_PORTRAIT                           =0
HPDF_PAGE_LANDSCAPE                          =1
HPDF_PageDirection                           =c_int
#enum ends




#enum starts
HPDF_ENCODER_TYPE_SINGLE_BYTE                =0
HPDF_ENCODER_TYPE_DOUBLE_BYTE                =1
HPDF_ENCODER_TYPE_UNINITIALIZED              =2
HPDF_ENCODER_UNKNOWN                         =3
HPDF_EncoderType                             =c_int
#enum ends




#enum starts
HPDF_BYTE_TYPE_SINGLE                        =0
HPDF_BYTE_TYPE_LEAD                          =1
HPDF_BYTE_TYPE_TRAIL                         =2
HPDF_BYTE_TYPE_UNKNOWN                       =3
HPDF_ByteType                                =c_int
#enum ends




#enum starts
HPDF_TALIGN_LEFT                             =0
HPDF_TALIGN_RIGHT                            =1
HPDF_TALIGN_CENTER                           =2
HPDF_TALIGN_JUSTIFY                          =3
HPDF_TextAlignment                           =c_int
#enum ends



###
## * << Haru Free PDF Library 2.0.0 >> -- ext_gstate_demo.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *
from haru.hpdf_errorcode import *


@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: %s, detail_no=%u\n", error_detail[error_no],
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)


def draw_circles (page, description, x, y):
    HPDF_Page_SetLineWidth (page, 1.0)
    HPDF_Page_SetRGBStroke (page, 0.0, 0.0, 0.0)
    HPDF_Page_SetRGBFill (page, 1.0, 0.0, 0.0)
    HPDF_Page_Circle (page, x + 40, y + 40, 40)
    HPDF_Page_ClosePathFillStroke (page)
    HPDF_Page_SetRGBFill (page, 0.0, 1.0, 0.0)
    HPDF_Page_Circle (page, x + 100, y + 40, 40)
    HPDF_Page_ClosePathFillStroke (page)
    HPDF_Page_SetRGBFill (page, 0.0, 0.0, 1.0)
    HPDF_Page_Circle (page, x + 70, y + 74.64, 40)
    HPDF_Page_ClosePathFillStroke (page)

    HPDF_Page_SetRGBFill (page, 0.0, 0.0, 0.0)
    HPDF_Page_BeginText (page)
    HPDF_Page_TextOut (page, x + 0.0, y + 130.0, description)
    HPDF_Page_EndText (page)


def main ():
    global pdf

    PAGE_WIDTH = 600
    PAGE_HEIGHT = 900

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    pdf = HPDF_New (error_handler, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1

    hfont = HPDF_GetFont (pdf, "Helvetica-Bold", NULL)

    # add a new page object.
    page = HPDF_AddPage (pdf)

    HPDF_Page_SetFontAndSize (page, hfont, 10)

    HPDF_Page_SetHeight (page, PAGE_HEIGHT)
    HPDF_Page_SetWidth (page, PAGE_WIDTH)

    # normal
    HPDF_Page_GSave (page)
    draw_circles (page, "normal", 40.0, PAGE_HEIGHT - 170)
    HPDF_Page_GRestore (page)

    # transparency (0.8)
    HPDF_Page_GSave (page)
    gstate = HPDF_CreateExtGState (pdf)
    HPDF_ExtGState_SetAlphaFill (gstate, 0.8)
    HPDF_ExtGState_SetAlphaStroke (gstate, 0.8)
    HPDF_Page_SetExtGState (page, gstate)
    draw_circles (page, "alpha fill = 0.8", 230.0, PAGE_HEIGHT - 170)
    HPDF_Page_GRestore (page)

    # transparency (0.4)
    HPDF_Page_GSave (page)
    gstate = HPDF_CreateExtGState (pdf)
    HPDF_ExtGState_SetAlphaFill (gstate, 0.4)
    HPDF_Page_SetExtGState (page, gstate)
    draw_circles (page, "alpha fill = 0.4", 420.0, PAGE_HEIGHT - 170)
    HPDF_Page_GRestore (page)

    # blend-mode=HPDF_BM_MULTIPLY
    HPDF_Page_GSave (page)
    gstate = HPDF_CreateExtGState (pdf)
    HPDF_ExtGState_SetBlendMode (gstate, HPDF_BM_MULTIPLY)
    HPDF_Page_SetExtGState (page, gstate)
    draw_circles (page, "HPDF_BM_MULTIPLY", 40.0, PAGE_HEIGHT - 340)
    HPDF_Page_GRestore (page)

    # blend-mode=HPDF_BM_SCREEN
    HPDF_Page_GSave (page)
    gstate = HPDF_CreateExtGState (pdf)
    HPDF_ExtGState_SetBlendMode (gstate, HPDF_BM_SCREEN)
    HPDF_Page_SetExtGState (page, gstate)
    draw_circles (page, "HPDF_BM_SCREEN", 230.0, PAGE_HEIGHT - 340)
    HPDF_Page_GRestore (page)

    # blend-mode=HPDF_BM_OVERLAY
    HPDF_Page_GSave (page)
    gstate = HPDF_CreateExtGState (pdf)
    HPDF_ExtGState_SetBlendMode (gstate, HPDF_BM_OVERLAY)
    HPDF_Page_SetExtGState (page, gstate)
    draw_circles (page, "HPDF_BM_OVERLAY", 420.0, PAGE_HEIGHT - 340)
    HPDF_Page_GRestore (page)

    # blend-mode=HPDF_BM_DARKEN
    HPDF_Page_GSave (page)
    gstate = HPDF_CreateExtGState (pdf)
    HPDF_ExtGState_SetBlendMode (gstate, HPDF_BM_DARKEN)
    HPDF_Page_SetExtGState (page, gstate)
    draw_circles (page, "HPDF_BM_DARKEN", 40.0, PAGE_HEIGHT - 510)
    HPDF_Page_GRestore (page)

    # blend-mode=HPDF_BM_LIGHTEN
    HPDF_Page_GSave (page)
    gstate = HPDF_CreateExtGState (pdf)
    HPDF_ExtGState_SetBlendMode (gstate, HPDF_BM_LIGHTEN)
    HPDF_Page_SetExtGState (page, gstate)
    draw_circles (page, "HPDF_BM_LIGHTEN", 230.0, PAGE_HEIGHT - 510)
    HPDF_Page_GRestore (page)

    # blend-mode=HPDF_BM_COLOR_DODGE
    HPDF_Page_GSave (page)
    gstate = HPDF_CreateExtGState (pdf)
    HPDF_ExtGState_SetBlendMode (gstate, HPDF_BM_COLOR_DODGE)
    HPDF_Page_SetExtGState (page, gstate)
    draw_circles (page, "HPDF_BM_COLOR_DODGE", 420.0, PAGE_HEIGHT - 510)
    HPDF_Page_GRestore (page)


    # blend-mode=HPDF_BM_COLOR_BUM
    HPDF_Page_GSave (page)
    gstate = HPDF_CreateExtGState (pdf)
    HPDF_ExtGState_SetBlendMode (gstate, HPDF_BM_COLOR_BUM)
    HPDF_Page_SetExtGState (page, gstate)
    draw_circles (page, "HPDF_BM_COLOR_BUM", 40.0, PAGE_HEIGHT - 680)
    HPDF_Page_GRestore (page)

    # blend-mode=HPDF_BM_HARD_LIGHT
    HPDF_Page_GSave (page)
    gstate = HPDF_CreateExtGState (pdf)
    HPDF_ExtGState_SetBlendMode (gstate, HPDF_BM_HARD_LIGHT)
    HPDF_Page_SetExtGState (page, gstate)
    draw_circles (page, "HPDF_BM_HARD_LIGHT", 230.0, PAGE_HEIGHT - 680)
    HPDF_Page_GRestore (page)

    # blend-mode=HPDF_BM_SOFT_LIGHT
    HPDF_Page_GSave (page)
    gstate = HPDF_CreateExtGState (pdf)
    HPDF_ExtGState_SetBlendMode (gstate, HPDF_BM_SOFT_LIGHT)
    HPDF_Page_SetExtGState (page, gstate)
    draw_circles (page, "HPDF_BM_SOFT_LIGHT", 420.0, PAGE_HEIGHT - 680)
    HPDF_Page_GRestore (page)

    # blend-mode=HPDF_BM_DIFFERENCE
    HPDF_Page_GSave (page)
    gstate = HPDF_CreateExtGState (pdf)
    HPDF_ExtGState_SetBlendMode (gstate, HPDF_BM_DIFFERENCE)
    HPDF_Page_SetExtGState (page, gstate)
    draw_circles (page, "HPDF_BM_DIFFERENCE", 40.0, PAGE_HEIGHT - 850)
    HPDF_Page_GRestore (page)


    # blend-mode=HPDF_BM_EXCLUSHON
    HPDF_Page_GSave (page)
    gstate = HPDF_CreateExtGState (pdf)
    HPDF_ExtGState_SetBlendMode (gstate, HPDF_BM_EXCLUSHON)
    HPDF_Page_SetExtGState (page, gstate)
    draw_circles (page, "HPDF_BM_EXCLUSHON", 230.0, PAGE_HEIGHT - 850)
    HPDF_Page_GRestore (page)


    # save the document to a file
    HPDF_SaveToFile (pdf, fname)

    # clean up
    HPDF_Free (pdf)

    return 0

main()


###
## * << Haru Free PDF Library 2.0.0 >> -- ttfont_demo_jp.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *
from haru.hpdf_errorcode import *


@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: %s, detail_no=%u\n", error_detail[error_no],
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)


def main ():
    global  pdf

    if (len(sys.argv) < 2):
        printf ("ttfont_jp_demo <ttf-font-filename> [-E]\n")
        printf ("ttfont_jp_demo <ttc-font-filename> <index> [-E]\n")
        return 1

    try:
        f = open ("mbtext/sjis.txt", "rb")
    except:
        printf ("error: cannot open 'mbtext/sjis.txt'\n")
        return 1


    SAMP_TXT=f.read(2048)
    f.close()

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    pdf = HPDF_New (error_handler, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1


    # declaration for using Japanese encoding.
    HPDF_UseJPEncodings (pdf)

    HPDF_SetCompressionMode (pdf, HPDF_COMP_ALL)

    # load ttc file
    if len(sys.argv) == 4 and sys.argv[3]=="-E":
        detail_font_name = HPDF_LoadTTFontFromFile2 (pdf, sys.argv[1],
                int (sys.argv[2]), HPDF_TRUE)
    elif len(sys.argv) == 3 and sys.argv[2]=="-E":
        detail_font_name = HPDF_LoadTTFontFromFile (pdf, sys.argv[1], HPDF_TRUE)
    elif len(sys.argv) == 3:
        detail_font_name = HPDF_LoadTTFontFromFile2 (pdf, sys.argv[1],
                int (sys.argv[2]), HPDF_FALSE)
    else:
        detail_font_name = HPDF_LoadTTFontFromFile (pdf, sys.argv[1], HPDF_FALSE)

    # add a new page object.
    page = HPDF_AddPage (pdf)

    title_font = HPDF_GetFont (pdf, "Helvetica", NULL)

    detail_font = HPDF_GetFont (pdf, detail_font_name, "90msp-RKSJ-H")

    HPDF_Page_SetFontAndSize (page, title_font, 10)

    HPDF_Page_BeginText (page)

    # move the position of the text to top of the page.
    HPDF_Page_MoveTextPos(page, 10, 190)
    HPDF_Page_ShowText (page, detail_font_name)
    HPDF_Page_ShowText (page, " (")
    HPDF_Page_ShowText (page, HPDF_Font_GetEncodingName (detail_font))
    HPDF_Page_ShowText (page, ")")

    HPDF_Page_SetFontAndSize (page, detail_font, 15)
    HPDF_Page_MoveTextPos (page, 10, -20)
    HPDF_Page_ShowText (page, "abcdefghijklmnopqrstuvwxyz")
    HPDF_Page_MoveTextPos (page, 0, -20)
    HPDF_Page_ShowText (page, "ABCDEFGHIJKLMNOPQRSTUVWXYZ")
    HPDF_Page_MoveTextPos (page, 0, -20)
    HPDF_Page_ShowText (page, "1234567890")
    HPDF_Page_MoveTextPos (page, 0, -20)

    HPDF_Page_SetFontAndSize (page, detail_font, 10)
    HPDF_Page_ShowText (page, SAMP_TXT)
    HPDF_Page_MoveTextPos (page, 0, -18)

    HPDF_Page_SetFontAndSize (page, detail_font, 16)
    HPDF_Page_ShowText (page, SAMP_TXT)
    HPDF_Page_MoveTextPos (page, 0, -27)

    HPDF_Page_SetFontAndSize (page, detail_font, 23)
    HPDF_Page_ShowText (page, SAMP_TXT)
    HPDF_Page_MoveTextPos (page, 0, -36)

    HPDF_Page_SetFontAndSize (page, detail_font, 30)
    HPDF_Page_ShowText (page, SAMP_TXT)
    HPDF_Page_MoveTextPos (page, 0, -36)

    pw = HPDF_Page_TextWidth (page, SAMP_TXT)
    page_height = 210
    page_width = pw + 40

    HPDF_Page_SetWidth (page, page_width)
    HPDF_Page_SetHeight (page, page_height)

    # finish to print text.
    HPDF_Page_EndText (page)

    HPDF_Page_SetLineWidth (page, 0.5)

    HPDF_Page_MoveTo (page, 10, page_height - 25)
    HPDF_Page_LineTo (page, page_width - 10, page_height - 25)
    HPDF_Page_Stroke (page)

    HPDF_Page_MoveTo (page, 10, page_height - 85)
    HPDF_Page_LineTo (page, page_width - 10, page_height - 85)
    HPDF_Page_Stroke (page)

    HPDF_SaveToFile (pdf, fname)

    # clean up
    HPDF_Free (pdf)

    return 0

main()


#coding=utf-8
###
## * << Haru Free PDF Library 2.0.0 >> -- text_annotation.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *
from haru.hpdf_errorcode import *


@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: %s, detail_no=%u\n", error_detail[error_no],
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)


def main():
    rect1 =HPDF_Rect(50, 350, 150, 400)
    rect2 =HPDF_Rect(210, 350, 350, 400)
    rect3 =HPDF_Rect(50, 250, 150, 300)
    rect4 =HPDF_Rect(210, 250, 350, 300)
    rect5 =HPDF_Rect(50, 150, 150, 200)
    rect6 =HPDF_Rect(210, 150, 350, 200)
    rect7 =HPDF_Rect(50, 50, 150, 100)
    rect8 =HPDF_Rect(210, 50, 350, 100)

    global  pdf

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    pdf = HPDF_New (error_handler, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1

    # use Times-Roman font.
    font = HPDF_GetFont (pdf, "Times-Roman", "WinAnsiEncoding")

    page = HPDF_AddPage (pdf)

    HPDF_Page_SetWidth (page, 400)
    HPDF_Page_SetHeight (page, 500)

    HPDF_Page_BeginText (page)
    HPDF_Page_SetFontAndSize (page, font, 16)
    HPDF_Page_MoveTextPos (page, 130, 450)
    HPDF_Page_ShowText (page, "Annotation Demo")
    HPDF_Page_EndText (page)


    annot = HPDF_Page_CreateTextAnnot (page, rect1, "Annotation with Comment "
                "Icon. \n This annotation set to be opened initially.",
                NULL)

    HPDF_TextAnnot_SetIcon (annot, HPDF_ANNOT_ICON_COMMENT)
    HPDF_TextAnnot_SetOpened (annot, HPDF_TRUE)

    annot = HPDF_Page_CreateTextAnnot (page, rect2,
                "Annotation with Key Icon", NULL)
    HPDF_TextAnnot_SetIcon (annot, HPDF_ANNOT_ICON_PARAGRAPH)

    annot = HPDF_Page_CreateTextAnnot (page, rect3,
                "Annotation with Note Icon", NULL)
    HPDF_TextAnnot_SetIcon (annot, HPDF_ANNOT_ICON_NOTE)

    annot = HPDF_Page_CreateTextAnnot (page, rect4,
                "Annotation with Help Icon", NULL)
    HPDF_TextAnnot_SetIcon (annot, HPDF_ANNOT_ICON_HELP)

    annot = HPDF_Page_CreateTextAnnot (page, rect5,
                "Annotation with NewParagraph Icon", NULL)
    HPDF_TextAnnot_SetIcon (annot, HPDF_ANNOT_ICON_NEW_PARAGRAPH)

    annot = HPDF_Page_CreateTextAnnot (page, rect6,
                "Annotation with Paragraph Icon", NULL)
    HPDF_TextAnnot_SetIcon (annot, HPDF_ANNOT_ICON_PARAGRAPH)

    annot = HPDF_Page_CreateTextAnnot (page, rect7,
                "Annotation with Insert Icon", NULL)
    HPDF_TextAnnot_SetIcon (annot, HPDF_ANNOT_ICON_INSERT)

    encoding = HPDF_GetEncoder (pdf, "ISO8859-2")

    #HPDF_Page_CreateTextAnnot (page, rect8,
    #            "Annotation with ISO8859 text 釉罩棕?, encoding)

    HPDF_Page_SetFontAndSize (page, font, 11)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, rect1.left + 35, rect1.top - 20)
    HPDF_Page_ShowText (page, "Comment Icon.")
    HPDF_Page_EndText (page)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, rect2.left + 35, rect2.top - 20)
    HPDF_Page_ShowText (page, "Key Icon")
    HPDF_Page_EndText (page)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, rect3.left + 35, rect3.top - 20)
    HPDF_Page_ShowText (page, "Note Icon.")
    HPDF_Page_EndText (page)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, rect4.left + 35, rect4.top - 20)
    HPDF_Page_ShowText (page, "Help Icon")
    HPDF_Page_EndText (page)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, rect5.left + 35, rect5.top - 20)
    HPDF_Page_ShowText (page, "NewParagraph Icon")
    HPDF_Page_EndText (page)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, rect6.left + 35, rect6.top - 20)
    HPDF_Page_ShowText (page, "Paragraph Icon")
    HPDF_Page_EndText (page)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, rect7.left + 35, rect7.top - 20)
    HPDF_Page_ShowText (page, "Insert Icon")
    HPDF_Page_EndText (page)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, rect8.left + 35, rect8.top - 20)
    HPDF_Page_ShowText (page, "Text Icon(ISO8859-2 text)")
    HPDF_Page_EndText (page)


    # save the document to a file
    HPDF_SaveToFile (pdf, fname)

    # clean up
    HPDF_Free (pdf)

    return 0

main()


###
## * << Haru Free PDF Library 2.0.0 >> -- raw_image_demo.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *
from haru.hpdf_errorcode import *


@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: %s, detail_no=%u\n", error_detail[error_no],
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)


RAW_IMAGE_DATA=[
    0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xfc,
    0xff, 0xff, 0xff, 0xf8, 0xff, 0xff, 0xff, 0xf0,
    0xf3, 0xf3, 0xff, 0xe0, 0xf3, 0xf3, 0xff, 0xc0,
    0xf3, 0xf3, 0xff, 0x80, 0xf3, 0x33, 0xff, 0x00,
    0xf3, 0x33, 0xfe, 0x00, 0xf3, 0x33, 0xfc, 0x00,
    0xf8, 0x07, 0xf8, 0x00, 0xf8, 0x07, 0xf0, 0x00,
    0xfc, 0xcf, 0xe0, 0x00, 0xfc, 0xcf, 0xc0, 0x00,
    0xff, 0xff, 0x80, 0x00, 0xff, 0xff, 0x00, 0x00,
    0xff, 0xfe, 0x00, 0x00, 0xff, 0xfc, 0x00, 0x00,
    0xff, 0xf8, 0x0f, 0xe0, 0xff, 0xf0, 0x0f, 0xe0,
    0xff, 0xe0, 0x0c, 0x30, 0xff, 0xc0, 0x0c, 0x30,
    0xff, 0x80, 0x0f, 0xe0, 0xff, 0x00, 0x0f, 0xe0,
    0xfe, 0x00, 0x0c, 0x30, 0xfc, 0x00, 0x0c, 0x30,
    0xf8, 0x00, 0x0f, 0xe0, 0xf0, 0x00, 0x0f, 0xe0,
    0xe0, 0x00, 0x00, 0x00, 0xc0, 0x00, 0x00, 0x00,
    0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
]

def main ():
    global  pdf

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    pdf = HPDF_New (error_handler, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1

    HPDF_SetCompressionMode (pdf, HPDF_COMP_ALL)

    # create default-font
    font = HPDF_GetFont (pdf, "Helvetica", NULL)

    # add a new page object.
    page = HPDF_AddPage (pdf)

    HPDF_Page_SetWidth (page, 172)
    HPDF_Page_SetHeight (page, 80)

    HPDF_Page_BeginText (page)
    HPDF_Page_SetFontAndSize (page, font, 20)
    HPDF_Page_MoveTextPos (page, 220, HPDF_Page_GetHeight (page) - 70)
    HPDF_Page_ShowText (page, "RawImageDemo")
    HPDF_Page_EndText (page)

    # load RGB raw-image file.
    image = HPDF_LoadRawImageFromFile (pdf, "rawimage/32_32_rgb.dat",
            32, 32, HPDF_CS_DEVICE_RGB)

    x = 20
    y = 20

    # Draw image to the canvas. (normal-mode with actual size.)
    HPDF_Page_DrawImage (page, image, x, y, 32, 32)

    # load GrayScale raw-image file.
    image = HPDF_LoadRawImageFromFile (pdf, "rawimage/32_32_gray.dat",
            32, 32, HPDF_CS_DEVICE_GRAY)

    x = 70
    y = 20

    # Draw image to the canvas. (normal-mode with actual size.)
    HPDF_Page_DrawImage (page, image, x, y, 32, 32)

    # load GrayScale raw-image (1bit) file from memory.
    image = HPDF_LoadRawImageFromMem (pdf, RAW_IMAGE_DATA, 32, 32,
                HPDF_CS_DEVICE_GRAY, 1)

    x = 120
    y = 20

    # Draw image to the canvas. (normal-mode with actual size.)
    HPDF_Page_DrawImage (page, image, x, y, 32, 32)

    # save the document to a file
    HPDF_SaveToFile (pdf, fname)

    # clean up
    HPDF_Free (pdf)

    return 0

main()


###
## * << Haru Free PDF Library 2.0.0 >> -- character_map.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
## * usage character_map <encoding-name> <low-range-from> <low-range-to>
## *              <high-range-from> <high-range-to>
## * ex. character_map 90ms-RKSJ-V 0x80 0x
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *
from haru.hpdf_errorcode import *


@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: %s, detail_no=%u\n", error_detail[error_no],
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)

def draw_page(pdf, page, title_font, font, h_byte, l_byte):
    PAGE_WIDTH = 420
    CELL_HEIGHT = 20
    CELL_WIDTH = 20


    l_byte = int(l_byte / 16) * 16
    h_count = 16 - (l_byte / 16)
    page_height = 40 + 40 + (h_count + 1) * CELL_HEIGHT

    HPDF_Page_SetHeight (page, page_height)
    HPDF_Page_SetWidth (page, PAGE_WIDTH)

    HPDF_Page_SetFontAndSize (page, title_font, 10)

    ypos = h_count + 1
    while True:
        y = (ypos) * CELL_HEIGHT + 40

        HPDF_Page_MoveTo (page, 40, y)
        HPDF_Page_LineTo (page, 380, y)
        HPDF_Page_Stroke (page)
        if (ypos < h_count):
            buf=[None,None]

            buf[0] = 16 - ypos - 1
            if (buf[0] < 10):
                buf[0] += ord('0')
            else:
                buf[0] += (ord('A') - 10)
            buf[1] = 0
            buf=[i % 256 for i in buf]      #because buf is unsigned char *

            w = HPDF_Page_TextWidth (page, buf)
            HPDF_Page_BeginText (page)
            HPDF_Page_MoveTextPos (page, 40 + (20 - w) / 2, y + 5)
            HPDF_Page_ShowText (page, buf)
            HPDF_Page_EndText (page)


        if (ypos == 0):
            break

        ypos-=1


    for xpos in range(18):
        y = (h_count + 1) * CELL_HEIGHT + 40
        x = xpos * CELL_WIDTH + 40

        HPDF_Page_MoveTo (page, x, 40)
        HPDF_Page_LineTo (page, x, y)
        HPDF_Page_Stroke (page)

        if (xpos > 0 and xpos <= 16):
            buf=[None,None]

            buf[0] = xpos - 1
            if (buf[0] < 10):
                buf[0] += ord('0')
            else:
                buf[0] += (ord('A') - 10)
            buf[1] = 0
            buf=[i % 256 for i in buf]      #because buf is unsigned char *

            w = HPDF_Page_TextWidth(page, buf)
            HPDF_Page_BeginText(page)
            HPDF_Page_MoveTextPos(page, x + (20 - w) / 2,
                        h_count * CELL_HEIGHT + 45)
            HPDF_Page_ShowText(page, buf)
            HPDF_Page_EndText(page)


    HPDF_Page_SetFontAndSize (page, font, 15)

    ypos = h_count
    while True:
        y = (ypos - 1) * CELL_HEIGHT + 45

        for xpos in range(16):
            buf=[None for i in range(3)]

            x = xpos * CELL_WIDTH + 40 + CELL_WIDTH

            buf[0] = h_byte
            buf[1] = (16 - ypos) * 16 + xpos
            buf[2] = 0x00
            buf=[i % 256 for i in buf]      #because buf is unsigned char *

            w = HPDF_Page_TextWidth(page, buf)
            if (w > 0):
                HPDF_Page_BeginText(page)
                HPDF_Page_MoveTextPos(page, x + (20 - w) / 2, y)
                HPDF_Page_ShowText(page, buf)
                HPDF_Page_EndText(page)


        if (ypos == 0):
            break

        ypos-=1


def main ():
    global  pdf

    flg=[HPDF_UINT16(0) for i in range(256)]

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    if (len(sys.argv) < 3):
        printf ("usage: character_map <encoding-name> <font-name>\n")
        printf ('for example, character_map.py GBK-EUC-H SimHei,Bold')
        return 1

    pdf = HPDF_New (error_handler, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1


    # configure pdf-document (showing outline, compression enabled)
    HPDF_SetPageMode(pdf, HPDF_PAGE_MODE_USE_OUTLINE)
    HPDF_SetCompressionMode (pdf, HPDF_COMP_ALL)
    HPDF_SetPagesConfiguration (pdf, 10)

    HPDF_UseJPEncodings (pdf)
    HPDF_UseJPFonts (pdf)
    HPDF_UseKREncodings (pdf)
    HPDF_UseKRFonts (pdf)
    HPDF_UseCNSEncodings (pdf)
    HPDF_UseCNSFonts (pdf)
    HPDF_UseCNTEncodings (pdf)
    HPDF_UseCNTFonts (pdf)

    encoder = HPDF_GetEncoder (pdf, sys.argv[1])
    if (HPDF_Encoder_GetType (encoder) != HPDF_ENCODER_TYPE_DOUBLE_BYTE):
        printf ("error: %s is not cmap-encoder\n", sys.argv[1])
        HPDF_Free (pdf)
        return 1

    font = HPDF_GetFont (pdf, sys.argv[2], sys.argv[1])

    min_l = 255
    min_h = 256
    max_l = 0
    max_h = 0


    for i in range(256):
        for j in range(20, 256):
            buf=[None, None ,None]
            code = i * 256 + j

            buf[0] = i
            buf[1] = j
            buf[2] = 0

            btype = HPDF_Encoder_GetByteType (encoder, buf, 0)
            unicode = HPDF_Encoder_GetUnicode (encoder, code)

            if (btype == HPDF_BYTE_TYPE_LEAD and
                    unicode != 0x25A1):
                if (min_l > j):
                    min_l = j

                if (max_l < j):
                    max_l = j

                if (min_h > i):
                    min_h = i

                if (max_h < i):
                    max_h = i

                flg[i] = 1

    printf ("min_h=%04X max_h=%04X min_l=%04X max_l=%04X\n",
            min_h, max_h, min_l, max_l)

    # create outline root.
    root = HPDF_CreateOutline (pdf, NULL, sys.argv[1], NULL)
    HPDF_Outline_SetOpened (root, HPDF_TRUE)

    for i in range(256):
        if (flg[i]):
            page = HPDF_AddPage (pdf)
            title_font = HPDF_GetFont (pdf, "Helvetica", NULL)

            buf="0x%04X-0x%04X" %(
                                  (i * 256 + min_l),
                                  (i * 256 + max_l)
                                 )

            outline = HPDF_CreateOutline (pdf, root, buf, NULL)
            dst = HPDF_Page_CreateDestination (page)
            HPDF_Outline_SetDestination(outline, dst)

            draw_page (pdf, page, title_font, font, i, min_l)

            buf="%s (%s) 0x%04X-0x%04X" %(
                        sys.argv[1],
                        sys.argv[2],
                        (i * 256 + min_l),
                        (i * 256 + max_l)
                        )
            HPDF_Page_SetFontAndSize (page, title_font, 10)
            HPDF_Page_BeginText (page)
            HPDF_Page_MoveTextPos (page, 40, HPDF_Page_GetHeight (page) - 35)
            HPDF_Page_ShowText (page, buf)
            HPDF_Page_EndText (page)

    HPDF_SaveToFile (pdf, fname)
    HPDF_Free (pdf)

    return 0

main()


#coding=ISO8859-2

###
## * << Haru Free PDF Library 2.0.0 >> -- outline_demo.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *
from haru.hpdf_errorcode import *

@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: %s, detail_no=%u\n", error_detail[error_no],
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)


def print_page  (page,  page_num):

    HPDF_Page_SetWidth (page, 800)
    HPDF_Page_SetHeight (page, 800)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, 30, 740)

    buf="Page:%d" % page_num


    HPDF_Page_ShowText (page, buf)
    HPDF_Page_EndText (page)


def main():
    global pdf
    page=[None for i in range(4)]
    outline=[None for i in range(4)]

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    pdf = HPDF_New (error_handler, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1

    # create default-font
    font = HPDF_GetFont (pdf, "Helvetica", NULL)

    # Set page mode to use outlines.
    HPDF_SetPageMode(pdf, HPDF_PAGE_MODE_USE_OUTLINE)

    # Add 3 pages to the document.
    page[0] = HPDF_AddPage (pdf)
    HPDF_Page_SetFontAndSize (page[0], font, 30)
    print_page(page[0], 1)

    page[1] = HPDF_AddPage (pdf)
    HPDF_Page_SetFontAndSize (page[1], font, 30)
    print_page(page[1], 2)

    page[2] = HPDF_AddPage (pdf)
    HPDF_Page_SetFontAndSize (page[2], font, 30)
    print_page(page[2], 3)

    # create outline root.
    root = HPDF_CreateOutline (pdf, NULL, "OutlineRoot", NULL)
    HPDF_Outline_SetOpened (root, HPDF_TRUE)

    outline[0] = HPDF_CreateOutline (pdf, root, "page1", NULL)
    outline[1] = HPDF_CreateOutline (pdf, root, "page2", NULL)

    # create outline with test which is ISO8859-2 encoding
    #outline[2] = HPDF_CreateOutline (pdf, root, "ISO8859-2 text 釉罩棕?,
    #                HPDF_GetEncoder (pdf, "ISO8859-2"))

    # create destination objects on each pages
    # and link it to outline items.

    dst = HPDF_Page_CreateDestination (page[0])
    HPDF_Destination_SetXYZ(dst, 0, HPDF_Page_GetHeight(page[0]), 1)
    HPDF_Outline_SetDestination(outline[0], dst)
    #HPDF_Catalog_SetOpenAction(dst)

    dst = HPDF_Page_CreateDestination (page[1])
    HPDF_Destination_SetXYZ(dst, 0, HPDF_Page_GetHeight(page[1]), 1)
    HPDF_Outline_SetDestination(outline[1], dst)

    dst = HPDF_Page_CreateDestination (page[2])
    HPDF_Destination_SetXYZ(dst, 0, HPDF_Page_GetHeight(page[2]), 1)
    HPDF_Outline_SetDestination(outline[2], dst)

    # save the document to a file
    HPDF_SaveToFile (pdf, fname)

    # clean up
    HPDF_Free (pdf)

    return 0

main()


###
## * << Haru Free PDF Library 2.0.0 >> -- line_demo.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *
from haru.hpdf_errorcode import *


@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: %s, detail_no=%u\n", error_detail[error_no],
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)


def draw_line  (page, x, y, label):
    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, x, y - 10)
    HPDF_Page_ShowText (page, label)
    HPDF_Page_EndText (page)

    HPDF_Page_MoveTo (page, x, y - 15)
    HPDF_Page_LineTo (page, x + 220, y - 15)
    HPDF_Page_Stroke (page)

def draw_line2  (page, x, y, label):
    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, x, y)
    HPDF_Page_ShowText (page, label)
    HPDF_Page_EndText (page)

    HPDF_Page_MoveTo (page, x + 30, y - 25)
    HPDF_Page_LineTo (page, x + 160, y - 25)
    HPDF_Page_Stroke (page)


def draw_rect (page,  x, y, label):
    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, x, y - 10)
    HPDF_Page_ShowText (page, label)
    HPDF_Page_EndText (page)

    HPDF_Page_Rectangle(page, x, y - 40, 220, 25)

def main ():
    global pdf

    page_title = "Line Example"

    DASH_MODE1= [3]
    DASH_MODE2= [3, 7]
    DASH_MODE3= [8, 7, 2, 7]

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    pdf = HPDF_New (error_handler, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1



    # create default-font
    font = HPDF_GetFont (pdf, "Helvetica", NULL)

    # add a new page object.
    page = HPDF_AddPage (pdf)

    # print the lines of the page.
    HPDF_Page_SetLineWidth (page, 1)
    HPDF_Page_Rectangle (page, 50, 50, HPDF_Page_GetWidth(page) - 100,
                HPDF_Page_GetHeight (page) - 110)
    HPDF_Page_Stroke (page)

    # print the title of the page (with positioning center).
    HPDF_Page_SetFontAndSize (page, font, 24)
    tw = HPDF_Page_TextWidth (page, page_title)
    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, (HPDF_Page_GetWidth(page) - tw) / 2,
                HPDF_Page_GetHeight (page) - 50)
    HPDF_Page_ShowText (page, page_title)
    HPDF_Page_EndText (page)

    HPDF_Page_SetFontAndSize (page, font, 10)

    # Draw various widths of lines.
    HPDF_Page_SetLineWidth (page, 0)
    draw_line (page, 60, 770, "line width = 0")

    HPDF_Page_SetLineWidth (page, 1.0)
    draw_line (page, 60, 740, "line width = 1.0")

    HPDF_Page_SetLineWidth (page, 2.0)
    draw_line (page, 60, 710, "line width = 2.0")

    # Line dash pattern
    HPDF_Page_SetLineWidth (page, 1.0)

    HPDF_Page_SetDash (page, DASH_MODE1, 1, 1)
    draw_line (page, 60, 680, "dash_ptn=[3], phase=1 -- "
                "2 on, 3 off, 3 on...")

    HPDF_Page_SetDash (page, DASH_MODE2, 2, 2)
    draw_line (page, 60, 650, "dash_ptn=[7, 3], phase=2 -- "
                "5 on 3 off, 7 on,...")

    HPDF_Page_SetDash (page, DASH_MODE3, 4, 0)
    draw_line (page, 60, 620, "dash_ptn=[8, 7, 2, 7], phase=0")

    HPDF_Page_SetDash (page, NULL, 0, 0)

    HPDF_Page_SetLineWidth (page, 30)
    HPDF_Page_SetRGBStroke (page, 0.0, 0.5, 0.0)

    # Line Cap Style
    HPDF_Page_SetLineCap (page, HPDF_BUTT_END)
    draw_line2 (page, 60, 570, "PDF_BUTT_END")

    HPDF_Page_SetLineCap (page, HPDF_ROUND_END)
    draw_line2 (page, 60, 505, "PDF_ROUND_END")

    HPDF_Page_SetLineCap (page, HPDF_PROJECTING_SQUARE_END)
    draw_line2 (page, 60, 440, "PDF_PROJECTING_SCUARE_END")

    # Line Join Style
    HPDF_Page_SetLineWidth (page, 30)
    HPDF_Page_SetRGBStroke (page, 0.0, 0.0, 0.5)

    HPDF_Page_SetLineJoin (page, HPDF_MITER_JOIN)
    HPDF_Page_MoveTo (page, 120, 300)
    HPDF_Page_LineTo (page, 160, 340)
    HPDF_Page_LineTo (page, 200, 300)
    HPDF_Page_Stroke (page)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, 60, 360)
    HPDF_Page_ShowText (page, "PDF_MITER_JOIN")
    HPDF_Page_EndText (page)

    HPDF_Page_SetLineJoin (page, HPDF_ROUND_JOIN)
    HPDF_Page_MoveTo (page, 120, 195)
    HPDF_Page_LineTo (page, 160, 235)
    HPDF_Page_LineTo (page, 200, 195)
    HPDF_Page_Stroke (page)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, 60, 255)
    HPDF_Page_ShowText (page, "PDF_ROUND_JOIN")
    HPDF_Page_EndText (page)

    HPDF_Page_SetLineJoin (page, HPDF_BEVEL_JOIN)
    HPDF_Page_MoveTo (page, 120, 90)
    HPDF_Page_LineTo (page, 160, 130)
    HPDF_Page_LineTo (page, 200, 90)
    HPDF_Page_Stroke (page)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, 60, 150)
    HPDF_Page_ShowText (page, "PDF_BEVEL_JOIN")
    HPDF_Page_EndText (page)

    # Draw Rectangle
    HPDF_Page_SetLineWidth (page, 2)
    HPDF_Page_SetRGBStroke (page, 0, 0, 0)
    HPDF_Page_SetRGBFill (page, 0.75, 0.0, 0.0)

    draw_rect (page, 300, 770, "Stroke")
    HPDF_Page_Stroke (page)

    draw_rect (page, 300, 720, "Fill")
    HPDF_Page_Fill (page)

    draw_rect (page, 300, 670, "Fill then Stroke")
    HPDF_Page_FillStroke (page)

    # Clip Rect
    HPDF_Page_GSave (page);  # Save the current graphic state
    draw_rect (page, 300, 620, "Clip Rectangle")
    HPDF_Page_Clip (page)
    HPDF_Page_Stroke (page)
    HPDF_Page_SetFontAndSize (page, font, 13)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, 290, 600)
    HPDF_Page_SetTextLeading (page, 12)
    HPDF_Page_ShowText (page,
                "Clip Clip Clip Clip Clip Clipi Clip Clip Clip")
    HPDF_Page_ShowTextNextLine (page,
                "Clip Clip Clip Clip Clip Clip Clip Clip Clip")
    HPDF_Page_ShowTextNextLine (page,
                "Clip Clip Clip Clip Clip Clip Clip Clip Clip")
    HPDF_Page_EndText (page)
    HPDF_Page_GRestore (page)

    # Curve Example(CurveTo2)
    x = 330
    y = 440
    x1 = 430
    y1 = 530
    x2 = 480
    y2 = 470
    x3 = 480
    y3 = 90

    HPDF_Page_SetRGBFill (page, 0, 0, 0)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, 300, 540)
    HPDF_Page_ShowText (page, "CurveTo2(x1, y1, x2. y2)")
    HPDF_Page_EndText (page)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, x + 5, y - 5)
    HPDF_Page_ShowText (page, "Current point")
    HPDF_Page_MoveTextPos (page, x1 - x, y1 - y)
    HPDF_Page_ShowText (page, "(x1, y1)")
    HPDF_Page_MoveTextPos (page, x2 - x1, y2 - y1)
    HPDF_Page_ShowText (page, "(x2, y2)")
    HPDF_Page_EndText (page)

    HPDF_Page_SetDash (page, DASH_MODE1, 1, 0)

    HPDF_Page_SetLineWidth (page, 0.5)
    HPDF_Page_MoveTo (page, x1, y1)
    HPDF_Page_LineTo (page, x2, y2)
    HPDF_Page_Stroke (page)

    HPDF_Page_SetDash (page, NULL, 0, 0)

    HPDF_Page_SetLineWidth (page, 1.5)

    HPDF_Page_MoveTo (page, x, y)
    HPDF_Page_CurveTo2 (page, x1, y1, x2, y2)
    HPDF_Page_Stroke (page)

    # Curve Example(CurveTo3)
    y -= 150
    y1 -= 150
    y2 -= 150

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, 300, 390)
    HPDF_Page_ShowText (page, "CurveTo3(x1, y1, x2. y2)")
    HPDF_Page_EndText (page)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, x + 5, y - 5)
    HPDF_Page_ShowText (page, "Current point")
    HPDF_Page_MoveTextPos (page, x1 - x, y1 - y)
    HPDF_Page_ShowText (page, "(x1, y1)")
    HPDF_Page_MoveTextPos (page, x2 - x1, y2 - y1)
    HPDF_Page_ShowText (page, "(x2, y2)")
    HPDF_Page_EndText (page)

    HPDF_Page_SetDash (page, DASH_MODE1, 1, 0)

    HPDF_Page_SetLineWidth (page, 0.5)
    HPDF_Page_MoveTo (page, x, y)
    HPDF_Page_LineTo (page, x1, y1)
    HPDF_Page_Stroke (page)

    HPDF_Page_SetDash (page, NULL, 0, 0)

    HPDF_Page_SetLineWidth (page, 1.5)
    HPDF_Page_MoveTo (page, x, y)
    HPDF_Page_CurveTo3 (page, x1, y1, x2, y2)
    HPDF_Page_Stroke (page)

    # Curve Example(CurveTo)
    y -= 150
    y1 -= 160
    y2 -= 130
    x2 += 10

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, 300, 240)
    HPDF_Page_ShowText (page, "CurveTo(x1, y1, x2. y2, x3, y3)")
    HPDF_Page_EndText (page)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, x + 5, y - 5)
    HPDF_Page_ShowText (page, "Current point")
    HPDF_Page_MoveTextPos (page, x1 - x, y1 - y)
    HPDF_Page_ShowText (page, "(x1, y1)")
    HPDF_Page_MoveTextPos (page, x2 - x1, y2 - y1)
    HPDF_Page_ShowText (page, "(x2, y2)")
    HPDF_Page_MoveTextPos (page, x3 - x2, y3 - y2)
    HPDF_Page_ShowText (page, "(x3, y3)")
    HPDF_Page_EndText (page)

    HPDF_Page_SetDash (page, DASH_MODE1, 1, 0)

    HPDF_Page_SetLineWidth (page, 0.5)
    HPDF_Page_MoveTo (page, x, y)
    HPDF_Page_LineTo (page, x1, y1)
    HPDF_Page_Stroke (page)
    HPDF_Page_MoveTo (page, x2, y2)
    HPDF_Page_LineTo (page, x3, y3)
    HPDF_Page_Stroke (page)

    HPDF_Page_SetDash (page, NULL, 0, 0)

    HPDF_Page_SetLineWidth (page, 1.5)
    HPDF_Page_MoveTo (page, x, y)
    HPDF_Page_CurveTo (page, x1, y1, x2, y2, x3, y3)
    HPDF_Page_Stroke (page)

    # save the document to a file
    HPDF_SaveToFile (pdf, fname)

    # clean up
    HPDF_Free (pdf)

    return 0


main()


###
## * << Haru Free PDF Library 2.0.0 >> -- chfont_demo.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *
from haru.hpdf_errorcode import *
from grid_sheet import *

@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: %s, detail_no=%u\n", error_detail[error_no],
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)

def main ():
    global  pdf

    if (len(sys.argv) < 4):
        printf ("chfont_demo <cp936-ttc-font-file-name> "
                "<cp936-index> <cp932-ttc-font-file-name> <cp932-index>\n")
        return 1


    fname="mbtext/%s"% "cp932.txt"
    cp932 = open (fname, "rb")
    if (not cp932):
        printf ("error: cannot open cp932.txt\n")
        return 1

    fname= "mbtext/%s" % "cp936.txt"
    cp936 = open (fname, "rb")
    if (not cp936):
        printf ("error: cannot open cp936.txt\n")
        return 1

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    pdf = HPDF_New (error_handler, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1


    HPDF_SetCompressionMode (pdf, HPDF_COMP_ALL)
    HPDF_UseJPEncodings (pdf)
    HPDF_UseCNSEncodings (pdf)

    fcp936_name = HPDF_LoadTTFontFromFile2 (pdf, sys.argv[1], int(sys.argv[2]),
            HPDF_TRUE)
    fcp932_name = HPDF_LoadTTFontFromFile2 (pdf, sys.argv[3], int(sys.argv[4]),
            HPDF_TRUE)

    # add a new page object.
    page = HPDF_AddPage (pdf)

    HPDF_Page_SetHeight (page, 300)
    HPDF_Page_SetWidth (page, 550)

    fcp936 = HPDF_GetFont (pdf, fcp936_name, "GBK-EUC-H")
    fcp932 = HPDF_GetFont (pdf, fcp932_name, "90ms-RKSJ-H")

    print_grid  (pdf, page)

    HPDF_Page_SetTextLeading (page, 20)

    HPDF_Page_BeginText (page)
    HPDF_Page_MoveTextPos (page, 50, 250)
    HPDF_Page_SetTextLeading (page, 25)

    buf=cp936.read(1024)
    while buf:
        HPDF_Page_SetFontAndSize (page, fcp936, 18)
        buf ='%s\0' % buf
        HPDF_Page_ShowText (page, buf)

        buf=cp936.read(1024)
        if buf:
            HPDF_Page_SetFontAndSize (page, fcp932, 18)
            buf ='%s\0' % buf
            HPDF_Page_ShowText (page, buf)

        HPDF_Page_MoveToNextLine (page)

    # save the document to a file
    HPDF_SaveToFile (pdf, fname)

    # clean up
    HPDF_Free (pdf)

    cp936.close ()
    cp932.close ()

    return 0

main()


###
## * << Haru Free PDF Library 2.0.0 >> -- grid_sheet.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *
from haru.hpdf_errorcode import *

@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: %s, detail_no=%u\n", error_detail[error_no],
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)


def print_grid  (pdf, page):
    height = HPDF_Page_GetHeight (page)
    width = HPDF_Page_GetWidth (page)
    font = HPDF_GetFont (pdf, "Helvetica", NULL)

    HPDF_Page_SetFontAndSize (page, font, 5)
    HPDF_Page_SetGrayFill (page, 0.5)
    HPDF_Page_SetGrayStroke (page, 0.8)

    # Draw horizontal lines
    y = 0
    while (y < height):
        if (y % 10 == 0):
            HPDF_Page_SetLineWidth (page, 0.5)
        else:
            if (HPDF_Page_GetLineWidth (page) != 0.25):
                HPDF_Page_SetLineWidth (page, 0.25)


        HPDF_Page_MoveTo (page, 0, y)
        HPDF_Page_LineTo (page, width, y)
        HPDF_Page_Stroke (page)

        if (y % 10 == 0 and y > 0):
            HPDF_Page_SetGrayStroke (page, 0.5)

            HPDF_Page_MoveTo (page, 0, y)
            HPDF_Page_LineTo (page, 5, y)
            HPDF_Page_Stroke (page)

            HPDF_Page_SetGrayStroke (page, 0.8)


        y += 5



    # Draw vertical lines
    x = 0
    while (x < width):
        if (x % 10 == 0):
            HPDF_Page_SetLineWidth (page, 0.5)
        else:
            if (HPDF_Page_GetLineWidth (page) != 0.25):
                HPDF_Page_SetLineWidth (page, 0.25)


        HPDF_Page_MoveTo (page, x, 0)
        HPDF_Page_LineTo (page, x, height)
        HPDF_Page_Stroke (page)

        if (x % 50 == 0 and x > 0):
            HPDF_Page_SetGrayStroke (page, 0.5)

            HPDF_Page_MoveTo (page, x, 0)
            HPDF_Page_LineTo (page, x, 5)
            HPDF_Page_Stroke (page)

            HPDF_Page_MoveTo (page, x, height)
            HPDF_Page_LineTo (page, x, height - 5)
            HPDF_Page_Stroke (page)

            HPDF_Page_SetGrayStroke (page, 0.8)


        x += 5


    # Draw horizontal text
    y = 0
    while (y < height):
        if (y % 10 == 0 and y > 0):

            HPDF_Page_BeginText (page)
            HPDF_Page_MoveTextPos (page, 5, y - 2)

            buf="%u" % y

            HPDF_Page_ShowText (page, buf)
            HPDF_Page_EndText (page)

        y += 5


    # Draw vertical text
    x = 0
    while (x < width):
        if (x % 50 == 0 and x > 0):

            HPDF_Page_BeginText (page)
            HPDF_Page_MoveTextPos (page, x, 5)

            buf="%u" % x

            HPDF_Page_ShowText (page, buf)
            HPDF_Page_EndText (page)

            HPDF_Page_BeginText (page)
            HPDF_Page_MoveTextPos (page, x, height - 10)
            HPDF_Page_ShowText (page, buf)
            HPDF_Page_EndText (page)

        x += 5

    HPDF_Page_SetGrayFill (page, 0)
    HPDF_Page_SetGrayStroke (page, 0)


def main():
    global pdf

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    pdf = HPDF_New (error_handler, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1

    # add a new page object.
    page = HPDF_AddPage (pdf)

    HPDF_Page_SetHeight (page, 600)
    HPDF_Page_SetWidth (page, 400)

    print_grid  (pdf, page)


    # save the document to a file
    HPDF_SaveToFile (pdf, fname)

    # clean up
    HPDF_Free (pdf)

    return 0

if __name__=='__main__':
    main()


__all__=['print_grid']


###
## * << Haru Free PDF Library 2.0.0 >> -- png_demo.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *
from haru.hpdf_errorcode import *

@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: %s, detail_no=%u\n", error_detail[error_no],
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)

def draw_image (pdf, filename, x, y, text):
    page = HPDF_GetCurrentPage (pdf)

    filename1= "pngsuite/%s" % filename

    image = HPDF_LoadPngImageFromFile (pdf, filename1)

    # Draw image to the canvas.
    HPDF_Page_DrawImage (page, image, x, y, HPDF_Image_GetWidth (image),
                    HPDF_Image_GetHeight (image))

    # Print the text.
    HPDF_Page_BeginText (page)
    HPDF_Page_SetTextLeading (page, 16)
    HPDF_Page_MoveTextPos (page, x, y)
    HPDF_Page_ShowTextNextLine (page, filename)
    HPDF_Page_ShowTextNextLine (page, text)
    HPDF_Page_EndText (page)


def main():
    global pdf

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    pdf = HPDF_New (error_handler, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1

    HPDF_SetCompressionMode (pdf, HPDF_COMP_ALL)

    # create default-font
    font = HPDF_GetFont (pdf, "Helvetica", NULL)

    # add a new page object.
    page = HPDF_AddPage (pdf)

    HPDF_Page_SetWidth (page, 550)
    HPDF_Page_SetHeight (page, 650)

    dst = HPDF_Page_CreateDestination (page)
    HPDF_Destination_SetXYZ (dst, 0, HPDF_Page_GetHeight (page), 1)
    HPDF_SetOpenAction(pdf, dst)

    HPDF_Page_BeginText (page)
    HPDF_Page_SetFontAndSize (page, font, 20)
    HPDF_Page_MoveTextPos (page, 220, HPDF_Page_GetHeight (page) - 70)
    HPDF_Page_ShowText (page, "PngDemo")
    HPDF_Page_EndText (page)

    HPDF_Page_SetFontAndSize (page, font, 12)

    draw_image (pdf, "basn0g01.png", 100, HPDF_Page_GetHeight (page) - 150,
                "1bit grayscale.")
    draw_image (pdf, "basn0g02.png", 200, HPDF_Page_GetHeight (page) - 150,
                "2bit grayscale.")
    draw_image (pdf, "basn0g04.png", 300, HPDF_Page_GetHeight (page) - 150,
                "4bit grayscale.")
    draw_image (pdf, "basn0g08.png", 400, HPDF_Page_GetHeight (page) - 150,
                "8bit grayscale.")

    draw_image (pdf, "basn2c08.png", 100, HPDF_Page_GetHeight (page) - 250,
                "8bit color.")
    draw_image (pdf, "basn2c16.png", 200, HPDF_Page_GetHeight (page) - 250,
                "16bit color.")

    draw_image (pdf, "basn3p01.png", 100, HPDF_Page_GetHeight (page) - 350,
                "1bit pallet.")
    draw_image (pdf, "basn3p02.png", 200, HPDF_Page_GetHeight (page) - 350,
                "2bit pallet.")
    draw_image (pdf, "basn3p04.png", 300, HPDF_Page_GetHeight (page) - 350,
                "4bit pallet.")
    draw_image (pdf, "basn3p08.png", 400, HPDF_Page_GetHeight (page) - 350,
                "8bit pallet.")

    draw_image (pdf, "basn4a08.png", 100, HPDF_Page_GetHeight (page) - 450,
                "8bit alpha.")
    draw_image (pdf, "basn4a16.png", 200, HPDF_Page_GetHeight (page) - 450,
                "16bit alpha.")

    draw_image (pdf, "basn6a08.png", 100, HPDF_Page_GetHeight (page) - 550,
                "8bit alpha.")
    draw_image (pdf, "basn6a16.png", 200, HPDF_Page_GetHeight (page) - 550,
                "16bit alpha.")

    # save the document to a file
    HPDF_SaveToFile (pdf, fname)

    # clean up
    HPDF_Free (pdf)

    return 0


if HPDF_NOPNGLIB:
    printf("WARNING: if you want to run this demo, \n"
           "make libhpdf with HPDF_USE_PNGLIB option.\n")
    sys.exit(1)
else:
    main()


###
## * << Haru Free PDF Library 2.0.0 >> -- jpfont_demo.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *
from haru.hpdf_errorcode import *


@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: %s, detail_no=%u\n", error_detail[error_no],
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)

def main():
    global pdf

    chinesefonts='''
SimSun
SimSun,Bold
SimSun,Italic
SimSun,BoldItalic
SimHei
SimHei,Bold
SimHei,Italic
SimHei,BoldItalic
'''
    chinesefonts=chinesefonts.split('\n')
    chinesefonts=[i for i in chinesefonts if i]

    detail_font=[]
    PAGE_HEIGHT = 210

    try:
        f = open ("mbtext/cp936.txt", "rb")
    except:
        printf ("error: cannot open 'mbtext/sjis.txt'\n")
        return 1

    samp_text=f.read(2048)
    f.close ()

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    pdf = HPDF_New (error_handler, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1


    # configure pdf-document to be compressed.
    HPDF_SetCompressionMode (pdf, HPDF_COMP_ALL)

    # declaration for using Japanese font, encoding.
    HPDF_UseCNSEncodings (pdf)
    HPDF_UseCNSFonts (pdf)


    for i in chinesefonts:
        detail_font.append( HPDF_GetFont (pdf, i, "GB-EUC-H"))

    # Set page mode to use outlines.
    HPDF_SetPageMode(pdf, HPDF_PAGE_MODE_USE_OUTLINE)

    # create outline root.
    root = HPDF_CreateOutline (pdf, NULL, "JP font demo", NULL)
    HPDF_Outline_SetOpened (root, HPDF_TRUE)

    for i in detail_font:

        # add a new page object.
        page = HPDF_AddPage (pdf)

        # create outline entry
        outline = HPDF_CreateOutline (pdf, root,
                HPDF_Font_GetFontName (i), NULL)
        dst = HPDF_Page_CreateDestination (page)
        HPDF_Outline_SetDestination(outline, dst)

        title_font = HPDF_GetFont (pdf, "Helvetica", NULL)
        HPDF_Page_SetFontAndSize (page, title_font, 10)

        HPDF_Page_BeginText (page)

        # move the position of the text to top of the page.
        HPDF_Page_MoveTextPos(page, 10, 190)
        HPDF_Page_ShowText (page, HPDF_Font_GetFontName (i))

        HPDF_Page_SetFontAndSize (page, i, 15)
        HPDF_Page_MoveTextPos (page, 10, -20)
        HPDF_Page_ShowText (page, "abcdefghijklmnopqrstuvwxyz")
        HPDF_Page_MoveTextPos (page, 0, -20)
        HPDF_Page_ShowText (page, "ABCDEFGHIJKLMNOPQRSTUVWXYZ")
        HPDF_Page_MoveTextPos (page, 0, -20)
        HPDF_Page_ShowText (page, "1234567890")
        HPDF_Page_MoveTextPos (page, 0, -20)

        HPDF_Page_SetFontAndSize (page, i, 10)
        HPDF_Page_ShowText (page, samp_text)
        HPDF_Page_MoveTextPos (page, 0, -18)

        HPDF_Page_SetFontAndSize (page, i, 16)
        HPDF_Page_ShowText (page, samp_text)
        HPDF_Page_MoveTextPos (page, 0, -27)

        HPDF_Page_SetFontAndSize (page, i, 23)
        HPDF_Page_ShowText (page, samp_text)
        HPDF_Page_MoveTextPos (page, 0, -36)

        HPDF_Page_SetFontAndSize (page, i, 30)
        HPDF_Page_ShowText (page, samp_text)

        p = HPDF_Page_GetCurrentTextPos (page)

        # finish to print text.
        HPDF_Page_EndText (page)

        HPDF_Page_SetLineWidth (page, 0.5)

        x_pos = 20
        for j in range(len (samp_text) // 2):
            HPDF_Page_MoveTo (page, x_pos, p.y - 10)
            HPDF_Page_LineTo (page, x_pos, p.y - 12)
            HPDF_Page_Stroke (page)
            x_pos = x_pos + 30

        HPDF_Page_SetWidth (page, p.x + 20)
        HPDF_Page_SetHeight (page, PAGE_HEIGHT)

        HPDF_Page_MoveTo (page, 10, PAGE_HEIGHT - 25)
        HPDF_Page_LineTo (page, p.x + 10, PAGE_HEIGHT - 25)
        HPDF_Page_Stroke (page)

        HPDF_Page_MoveTo (page, 10, PAGE_HEIGHT - 85)
        HPDF_Page_LineTo (page, p.x + 10, PAGE_HEIGHT - 85)
        HPDF_Page_Stroke (page)

        HPDF_Page_MoveTo (page, 10, p.y - 12)
        HPDF_Page_LineTo (page, p.x + 10, p.y - 12)
        HPDF_Page_Stroke (page)

    HPDF_SaveToFile (pdf, fname)

    # clean up
    HPDF_Free (pdf)

    return 0

main()


###
## * << Haru Free PDF Library 2.0.0 >> -- arc_demo.c
## *
## * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
## *
## * Permission to use, copy, modify, distribute and sell this software
## * and its documentation for any purpose is hereby granted without fee,
## * provided that the above copyright notice appear in all copies and
## * that both that copyright notice and this permission notice appear
## * in supporting documentation.
## * It is provided "as is" without express or implied warranty.
## *
##

## port to python by Li Jun
## http://groups.google.com/group/pythoncia

import os, sys

from ctypes import *
up=2
def setlibpath(up):
    import sys
    path=os.path.normpath(os.path.split(os.path.realpath(__file__))[0]+'\..'*up)
    if path not in sys.path:
        sys.path.append(path)

setlibpath(up)

from haru import *
from haru.c_func import *
from haru.hpdf_errorcode import *
from grid_sheet import *


@HPDF_Error_Handler(None, HPDF_UINT, HPDF_UINT, c_void_p)
def error_handler (error_no, detail_no, user_data):
    global pdf
    printf ("ERROR: %s, detail_no=%u\n", error_detail[error_no],
                detail_no)
    HPDF_Free (pdf)
    sys.exit(1)


def main ():
    global pdf

    fname=os.path.realpath(sys.argv[0])
    fname=fname[:fname.rfind('.')]+'.pdf'

    pdf = HPDF_New (error_handler, NULL)
    if (not pdf):
        printf ("error: cannot create PdfDoc object\n")
        return 1

    # add a new page object.
    page = HPDF_AddPage (pdf)

    HPDF_Page_SetHeight (page, 220)
    HPDF_Page_SetWidth (page, 200)

    # draw grid to the page
    print_grid  (pdf, page)

    # draw pie chart
    # *
    # *   A: 45% Red
    # *   B: 25% Blue
    # *   C: 15% green
    # *   D: other yellow


    # A
    HPDF_Page_SetRGBFill (page, 1.0, 0, 0)
    HPDF_Page_MoveTo (page, 100, 100)
    HPDF_Page_LineTo (page, 100, 180)
    HPDF_Page_Arc (page, 100, 100, 80, 0, 360 * 0.45)
    pos = HPDF_Page_GetCurrentPos (page)
    HPDF_Page_LineTo (page, 100, 100)
    HPDF_Page_Fill (page)

    # B
    HPDF_Page_SetRGBFill (page, 0, 0, 1.0)
    HPDF_Page_MoveTo (page, 100, 100)
    HPDF_Page_LineTo (page, pos.x, pos.y)
    HPDF_Page_Arc (page, 100, 100, 80, 360 * 0.45, 360 * 0.7)
    pos = HPDF_Page_GetCurrentPos (page)
    HPDF_Page_LineTo (page, 100, 100)
    HPDF_Page_Fill (page)

    # C
    HPDF_Page_SetRGBFill (page, 0, 1.0, 0)
    HPDF_Page_MoveTo (page, 100, 100)
    HPDF_Page_LineTo (page, pos.x, pos.y)
    HPDF_Page_Arc (page, 100, 100, 80, 360 * 0.7, 360 * 0.85)
    pos = HPDF_Page_GetCurrentPos (page)
    HPDF_Page_LineTo (page, 100, 100)
    HPDF_Page_Fill (page)

    # D
    HPDF_Page_SetRGBFill (page, 1.0, 1.0, 0)
    HPDF_Page_MoveTo (page, 100, 100)
    HPDF_Page_LineTo (page, pos.x, pos.y)
    HPDF_Page_Arc (page, 100, 100, 80, 360 * 0.85, 360)
    pos = HPDF_Page_GetCurrentPos (page)
    HPDF_Page_LineTo (page, 100, 100)
    HPDF_Page_Fill (page)

    # draw center circle
    HPDF_Page_SetGrayStroke (page, 0)
    HPDF_Page_SetGrayFill (page, 1)
    HPDF_Page_Circle (page, 100, 100, 30)
    HPDF_Page_Fill (page)

    # save the document to a file
    HPDF_SaveToFile (pdf, fname)

    # clean up
    HPDF_Free (pdf)

    return 0

main()